"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmeCalc = void 0;
const Log_1 = require("../logging/Log");
class SmeCalc {
    static fromFile(filePath) {
        //Placed here to not interfere with frontend apps
        const fs = require('fs-extra');
        try {
            Log_1.Log.logDebug("Getting SME calc for file " + filePath);
            const fileLength = fs.statSync(filePath).size;
            const s = 0;
            let m = Math.floor((fileLength / 2) - 512);
            if (m < 0) {
                m = 0;
            }
            let e = fileLength - 1024;
            if (e < 0) {
                e = 0;
            }
            const fd = fs.openSync(filePath, 'r');
            let buff = Buffer.alloc(1024);
            const blen = fs.readSync(fd, buff, 0, 1024, s);
            const start = this.sum(buff, blen);
            const mlen = fs.readSync(fd, buff, 0, 1024, m);
            const middle = this.sum(buff, mlen);
            const elen = fs.readSync(fd, buff, 0, 1024, e);
            const end = this.sum(buff, elen);
            fs.closeSync(fd);
            let startResult = start.toString(16).padStart(4, '0');
            let middleResult = middle.toString(16).padStart(4, '0');
            let endResult = end.toString(16).padStart(4, '0');
            return `H${startResult}.M${middleResult}.T${endResult}`.toUpperCase();
        }
        catch (err) {
            Log_1.Log.logError("error getting SME calc for file " + filePath + " - " + err);
            return undefined;
        }
    }
    static fromArrayBuffer(arrayBuffer) {
        try {
            const s = 0;
            let m = Math.floor((arrayBuffer.byteLength / 2) - 512);
            if (m < 0) {
                m = 0;
            }
            let e = arrayBuffer.byteLength - 1024;
            if (e < 0) {
                e = 0;
            }
            const start = this.sum(new Int8Array(arrayBuffer.slice(s, s + 1024)), 1024);
            const middle = this.sum(new Int8Array(arrayBuffer.slice(m, m + 1024)), 1024);
            const end = this.sum(new Int8Array(arrayBuffer.slice(e, e + 1024)), 1024);
            let startResult = start.toString(16).padStart(4, '0');
            let middleResult = middle.toString(16).padStart(4, '0');
            let endResult = end.toString(16).padStart(4, '0');
            return `H${startResult}.M${middleResult}.T${endResult}`.toUpperCase();
        }
        catch (err) {
            Log_1.Log.logError("error getting SME calc " + err);
            return undefined;
        }
    }
    static sum(buff, length) {
        let odd = 0;
        let even = 0;
        for (let i = 0; i < length; i++) {
            if ((i % 2) == 0) {
                odd ^= (buff[i] & 0xff);
            }
            else {
                even ^= (buff[i] & 0xff);
            }
        }
        return even | (odd << 8);
    }
}
exports.SmeCalc = SmeCalc;
