"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WavMeta = void 0;
const fs_1 = require("fs");
const _ = require("lodash");
class File {
}
;
class WavMeta {
    constructor(audioFile) {
        this.audioFile = audioFile;
        this.metaData = null;
    }
    getFourCC(buffer, offset) {
        return buffer.slice(offset, offset + 4).toString();
    }
    async getBuffer() {
        if (_.isArrayBuffer(this.audioFile)) {
            return Buffer.from(this.audioFile);
        }
        else if (!_.isString(this.audioFile)) {
            const blob = this.audioFile.slice(0, 2048);
            return Buffer.from(await blob.arrayBuffer());
        }
        else {
            const buffer = Buffer.alloc(2048);
            const fh = await fs_1.promises.open(this.audioFile, "r");
            const header = await fh.read(buffer, 0, 2048, 0);
            if (header.bytesRead > 40) {
                return header.buffer;
            }
        }
        return null;
    }
    async readFile() {
        const result = {};
        let buffer = await this.getBuffer();
        if (buffer != null) {
            const riff = this.getFourCC(buffer, 0);
            const chunkSize = buffer.readUInt32LE(4);
            const wave = this.getFourCC(buffer, 8);
            let byteRate = 0;
            let blockAlign = 0;
            let blockPos = 12;
            while (blockPos < 1024) {
                const block = this.getFourCC(buffer, blockPos);
                blockPos += 4;
                const blockSize = buffer.readUInt32LE(blockPos);
                blockPos += 4;
                if (block === 'fmt ') {
                    let pos = blockPos;
                    const audioFormat = buffer.readUInt16LE(pos);
                    if (audioFormat == 1) { // pcm
                        result['format'] = 'PCM';
                        pos += 2;
                        const numChannels = buffer.readUInt16LE(pos);
                        pos += 2;
                        const sampleRate = buffer.readUInt32LE(pos);
                        pos += 4;
                        byteRate = buffer.readUInt32LE(pos);
                        pos += 4;
                        blockAlign = buffer.readUInt16LE(pos);
                        pos += 2;
                        const bitsPerSample = buffer.readUInt16LE(pos);
                        pos += 2;
                        result['numChannels'] = numChannels;
                        result['sampleRate'] = sampleRate;
                        result['bitsPerSample'] = bitsPerSample;
                    }
                }
                else if (block === 'data') {
                    if (byteRate > 0) {
                        let duration = blockSize / byteRate;
                        let sampleCount = blockSize / blockAlign;
                        result['totalDuration'] = duration;
                        result['sampleCount'] = sampleCount;
                    }
                    break;
                }
                blockPos += blockSize;
            }
        }
        return result;
    }
    async getMetadata() {
        if (this.metaData == null) {
            this.metaData = await this.readFile();
        }
        return this.metaData;
    }
}
exports.WavMeta = WavMeta;
