"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ampp = void 0;
const _ = require("lodash");
const bTime_1 = require("../bTime");
class Ampp {
    static amppStartTimeToBDateTime(time) {
        return bTime_1.BDateTime.fromIso(time, "ndf25");
    }
    static amppTimeToBDateTime(time) {
        if (time) {
            return bTime_1.BDateTime.fromTimecode(time, "milli");
        }
    }
    static amppTimeToBDuration(time) {
        if (time) {
            return bTime_1.BDuration.fromTcString(time, "milli");
        }
    }
    static flatten(items) {
        return Ampp.flattenItems(items, '');
    }
    static flattenItems(items, groupId) {
        return _.flatMapDeep(items, (item) => {
            let result = [{
                    id: item.id,
                    name: item.name,
                    groupId: groupId,
                    payload: item.payload
                }];
            if (item.items?.length > 0) {
                result = result.concat(Ampp.flattenItems(item.items, groupId ? groupId + "|" + item.id : item.id));
            }
            return result;
        });
    }
    static unflatten(items) {
        return Ampp.unflattenItems(items, '');
    }
    static unflattenItems(items, id) {
        return _(items)
            .filter((item) => {
            const parentId = _.last(item.groupId.split("|"));
            return id === parentId;
        })
            .map((item) => {
            return _.omit({
                ...item,
                items: Ampp.unflattenItems(items, item.id)
            }, 'groupId');
        })
            .value();
    }
    static eventsToFlattened(events, onPrimaryToItem) {
        let items = [];
        for (const event of events) {
            const item = onPrimaryToItem(event);
            if (item) {
                items = items.concat(item);
            }
        }
        return items;
    }
    static flattenedToEvents(items, onItemToEvent) {
        let events = [];
        for (const item of items) {
            const event = onItemToEvent(item, events);
            if (event) {
                events = events.concat(event);
            }
        }
        return events;
    }
    static timeModeToStartMode(timeMode) {
        switch (timeMode) {
            case "Auto": {
                return "follow";
            }
            case "Manual": {
                return "manual";
            }
            default: {
                return "follow";
            }
        }
    }
    static timeModeToSecondaryStartMode(timeMode) {
        switch (timeMode) {
            case "StartPlus": {
                return "offsetFromParentStart";
            }
            default: {
                return "offsetFromParentStart";
            }
        }
    }
}
exports.Ampp = Ampp;
