"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BDateTimeCount = void 0;
const BDateTime_1 = require("./BDateTime");
const Counter_1 = require("./Counter");
const CounterMode_1 = require("./CounterMode");
class BDateTimeCount {
    static fromJson(jo) {
        const dateTime = BDateTime_1.BDateTime.fromJson(jo);
        const nowTime = BDateTime_1.BDateTime.fromJson(jo.counter.reference);
        const cm = new CounterMode_1.CounterMode(jo.counter.mode);
        const rate = jo.counter && jo.counter.rate || 1;
        return new BDateTimeCount(dateTime, nowTime, cm, rate);
    }
    constructor(bDateTime, nowTime, cm, rate, autoStop) {
        if (rate < 0) {
            rate = -rate;
        }
        this.bDateTime = bDateTime;
        this.counter = new Counter_1.Counter(cm, nowTime, rate, autoStop);
    }
    getBDateTime(nowTime) {
        return this.bDateTime.addFrames(this.counter.getOffset(nowTime));
    }
    toJson() {
        const result = this.bDateTime.toJson();
        result.counter = this.counter.toJson();
        return result;
    }
    getTimeString(nowTime, offset) {
        let time = this.getBDateTime(nowTime);
        if (offset) {
            time = time.addDuration(offset);
        }
        return time.getTimeString();
    }
    getDateString(nowTime, offset) {
        let time = this.getBDateTime(nowTime);
        if (offset) {
            time = time.addDuration(offset);
        }
        return time.getDateString();
    }
    toString(nowTime) {
        return this.getBDateTime(nowTime).toString();
    }
    toLocalString(nowTime, offset) {
        return this.bDateTime.addFrames(this.counter.getOffset(nowTime) + offset.getFrames()).toString();
    }
    getAsCounter(nowTime, mode, rate = 1) {
        return new BDateTimeCount(this.getBDateTime(nowTime), nowTime, mode, rate);
    }
    equals(nowTime, other) {
        return this.getBDateTime(nowTime).equals(other.getBDateTime(nowTime));
    }
    getComponents(nowTime) {
        return this.getBDateTime(nowTime).getComponents();
    }
}
exports.BDateTimeCount = BDateTimeCount;
