"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BDurationCount = void 0;
const BDuration_1 = require("./BDuration");
const Counter_1 = require("./Counter");
const BDateTime_1 = require("./BDateTime");
const CounterMode_1 = require("./CounterMode");
class BDurationCount {
    static fromJson(jo) {
        const dur = BDuration_1.BDuration.fromJson(jo);
        const nowTime = BDateTime_1.BDateTime.fromJson(jo.counter.reference);
        const cm = new CounterMode_1.CounterMode(jo.counter.mode);
        const rate = jo.rate || 1;
        return new BDurationCount(dur, nowTime, cm, rate);
    }
    constructor(bDuration, nowTime, cm, rate) {
        if (rate < 0) {
            rate = -rate;
        }
        this.bDuration = bDuration;
        this.counter = new Counter_1.Counter(cm, nowTime, rate, bDuration);
    }
    getBDuration(nowTime) {
        return this.bDuration.addFrames(this.counter.getOffset(nowTime));
    }
    toJson() {
        const result = this.bDuration.toJson();
        result.counter = this.counter.toJson();
        return result;
    }
    toString(nowTime) {
        return this.getBDuration(nowTime).toString();
    }
    getAsCounter(nowTime, mode, rate) {
        return new BDurationCount(this.getBDuration(nowTime), nowTime, mode, rate);
    }
    equals(nowTime, other) {
        return this.getBDuration(nowTime).equals(other.getBDuration(nowTime));
    }
    getComponents(nowTime) {
        return this.getBDuration(nowTime).getComponents();
    }
}
exports.BDurationCount = BDurationCount;
