"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrameRate = void 0;
class FrameRate {
    static ratio(ratio) {
        switch (ratio) {
            case "1x1": return this.milli();
            case "24x1": return this.ndf24();
            case "25x1": return this.ndf25();
            case "30x1": return this.ndf30();
            case "50x1": return this.ndf50();
            case "60x1": return this.ndf60();
            case "24000x1001": return this.df24();
            case "30000x1001": return this.df30();
            case "60000x1001": return this.df60();
            default: return this.milli();
        }
    }
    static decimal(decimal, defaultFrameRate = this.milli()) {
        if (typeof decimal === "number") {
            decimal = decimal.toFixed(3);
        }
        switch (decimal) {
            case '1000.000': return this.milli();
            case '24.000': return this.ndf24();
            case '25.000': return this.ndf25();
            case '30.000': return this.ndf30();
            case '50.000': return this.ndf50();
            case '60.000': return this.ndf60();
            case '23.976': return this.df24();
            case '29.970': return this.df30();
            case '59.940': return this.df60();
            default: return defaultFrameRate;
        }
    }
    constructor(fr) {
        this.fr = fr;
        switch (fr) {
            case "ndf25":
                {
                    this.framesInDay = 2160000;
                    this.framesPerSecond = 25;
                    this.usPerFrame = 40000;
                    this.separator = ":";
                }
                break;
            case "df30":
                {
                    this.framesInDay = 2589411;
                    this.framesPerSecond = (30.0 * 1000.0) / 1001.0;
                    this.usPerFrame = 33366;
                    this.separator = ";";
                }
                break;
            case "ndf30":
                {
                    this.framesInDay = 2592000;
                    this.framesPerSecond = 30;
                    this.usPerFrame = 1000000 / 30;
                    this.separator = ":";
                }
                break;
            case "ndf50":
                {
                    this.framesInDay = 4320000;
                    this.framesPerSecond = 50;
                    this.usPerFrame = 20000;
                    this.separator = ":";
                }
                break;
            case "df60":
                {
                    this.framesInDay = 5178821;
                    this.framesPerSecond = (60.0 * 1000.0) / 1001.0;
                    this.usPerFrame = 16683;
                    this.separator = ";";
                }
                break;
            case "ndf60":
                {
                    this.framesInDay = 5184000;
                    this.framesPerSecond = 60;
                    this.usPerFrame = 1000000 / 60;
                    this.separator = ":";
                }
                break;
            case "ndf24":
                {
                    this.framesInDay = 2073600;
                    this.framesPerSecond = 24;
                    this.usPerFrame = 41666;
                    this.separator = ":";
                }
                break;
            case "df24":
                {
                    this.framesInDay = 2071528;
                    this.framesPerSecond = (24.0 * 1000.0) / 1001.0;
                    this.usPerFrame = 41083;
                    this.separator = ";";
                }
                break;
            case "milli":
                {
                    this.framesInDay = 86400000;
                    this.framesPerSecond = 1000.0;
                    this.usPerFrame = 1000.0;
                    this.separator = ".";
                }
                break;
        }
    }
    getSeparator() {
        return this.separator;
    }
    getLabel() {
        return this.fr;
    }
    getFramesInDay() {
        return this.framesInDay;
    }
    getFramesPerSecond() {
        return this.framesPerSecond;
    }
    getUsPerFrame() {
        return this.usPerFrame;
    }
    getFrtype() {
        return this.fr;
    }
    ;
    static ndf25() {
        return new FrameRate("ndf25");
    }
    static ndf30() {
        return new FrameRate("ndf30");
    }
    static df30() {
        return new FrameRate("df30");
    }
    static ndf50() {
        return new FrameRate("ndf50");
    }
    static ndf60() {
        return new FrameRate("ndf60");
    }
    static df60() {
        return new FrameRate("df60");
    }
    static ndf24() {
        return new FrameRate("ndf24");
    }
    static df24() {
        return new FrameRate("df24");
    }
    static milli() {
        return new FrameRate("milli");
    }
}
exports.FrameRate = FrameRate;
