"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliCommandDescription = void 0;
const CliAutoCompleteParam_1 = require("./Params/CliAutoCompleteParam");
const Params_1 = require("./Params");
class CliCommandDescription {
    static make(command, description) {
        return new CliCommandDescription(command, description);
    }
    constructor(command, description) {
        this.command = command;
        this.description = description;
        this.parameters = {};
    }
    getCommand() {
        return this.command;
    }
    getDescription() {
        return this.description;
    }
    getParam(param) {
        return this.parameters[param];
    }
    getHelp() {
        const result = [];
        result.push(`${this.command.padEnd(15, " ")} ${this.description}`);
        for (const [name, param] of Object.entries(this.parameters)) {
            result.push(`${"".padEnd(20, " ")} ${param.toString()}`);
        }
        return result;
    }
    addStringParameter(name, description, required = false, def) {
        this.parameters[name] = new Params_1.CliStringParam(name, description, required, def);
        return this;
    }
    addIntegerParameter(name, description, required = false, def) {
        this.parameters[name] = new Params_1.CliIntegerParam(name, description, required, def);
        return this;
    }
    addEnumParameter(name, description, required, def, ...rest) {
        const options = rest.length === 1 ? rest[0] : rest;
        this.parameters[name] = new Params_1.CliEnumParam(name, description, required, def, options);
        return this;
    }
    addAutoCompleteParameter(name, description, required = false, def, autoComplete) {
        this.parameters[name] = new CliAutoCompleteParam_1.CliAutoCompleteParam(name, description, required, def, autoComplete);
        return this;
    }
    addFlag(name, description) {
        this.parameters[name] = new Params_1.CliFlagParam(name, description);
        return this;
    }
    addBooleanParameter(name, description, required = false, type, def) {
        this.parameters[name] = new Params_1.CliBooleanParam(name, description, required, type, def);
        return this;
    }
    toJson() {
        const result = {
            command: this.command,
            description: this.description,
            params: Object.values(this.parameters).map(p => p.toJson())
        };
        return result;
    }
}
exports.CliCommandDescription = CliCommandDescription;
