"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliParams = void 0;
const lodash_1 = require("lodash");
class CliParams {
    constructor() {
        this.params = {};
    }
    addParam(key, val) {
        this.params[key.replace("-", "").trim()] = val.trim();
        return this;
    }
    getStringParam(key) {
        return this.getStringParamDefault(key, "");
    }
    getStringParamDefault(key, defaultString) {
        return this.params[key] ?? defaultString;
    }
    getStringListParam(key) {
        return this.getStringParamDefault(key, null)?.split(",").map(v => v.trim());
    }
    getIntParam(key) {
        return this.getIntDefaultParam(key, -1);
    }
    getIntDefaultParam(key, defaultInt) {
        let result = defaultInt;
        if (this.params[key]) {
            try {
                result = Number.parseInt(this.params[key]);
            }
            catch (e) {
            }
        }
        return result;
    }
    getIntListParam(key) {
        return this.getStringListParam(key).map(v => parseInt(v));
    }
    hasParam(key) {
        return (0, lodash_1.has)(this.params, key);
    }
    hasParams(...keys) {
        for (const key of keys) {
            if (!(0, lodash_1.has)(this.params, key)) {
                return false;
            }
        }
        return true;
    }
}
exports.CliParams = CliParams;
