"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogLevelCommand = void 0;
const CliCommand_1 = require("../CliCommand");
const Log_1 = require("../../../logging/Log");
const _ = require("lodash");
const CliCommandDescription_1 = require("../CliCommandDescription");
class LogLevelCommand extends CliCommand_1.CliCommand {
    constructor() {
        super(CliCommandDescription_1.CliCommandDescription.make("ll", "Sets a custom log level while running")
            .addFlag("r", "Reset to configurred levels")
            .addFlag("f", "Turn on all log levels")
            .addEnumParameter("l", "The level to change", false, null, Object.values(Log_1.LogLevel).map(v => (0, Log_1.logLevelToDisplayName)(v)))
            .addBooleanParameter("s", "The specific state to set the log level to, if omitted toggles", false, "trueFalse"));
        this.processCommand = async (params, provider) => {
            const result = [];
            if (params.hasParam("r")) {
                result.push(`Resetting log level to configured values`);
                result.push(``);
                // Reset log levels
                if (provider.getLoadedConfiguration()?.getConfigData()?.logLevels) {
                    for (const [key, value] of _.entries(provider.getLoadedConfiguration().getConfigData().logLevels)) {
                        Log_1.Log.setLogLevel(Log_1.LogLevel[key], value);
                        result.push(`${(0, Log_1.logLevelToDisplayName)(Log_1.LogLevel[key]).padEnd(15, " ")}${value}`);
                    }
                }
            }
            else if (params.hasParam("f")) {
                result.push(`Changing Full Logging status`);
                result.push(``);
                // All log levels
                const state = params.getStringParam("s");
                if (!params.hasParam("s")) {
                    Log_1.Log.SIGUSR2Handler();
                    result.push(`${Log_1.Log.fullLogging ? "Enabling" : "Disabling"} full logging`);
                }
                else if (state === "true" || state === "false" || state === "t" || state === "f") {
                    Log_1.Log.SIGUSR2Handler(state === "true" || state === "t");
                    result.push(`${state === "true" || state === "t" ? "Enabling" : "Disabling"} full logging`);
                }
                else {
                    result.push("-s must be true or false");
                }
            }
            else if (params.hasParam("l")) {
                result.push(`Setting specific log levels`);
                result.push(``);
                // Specific log level
                const levels = params.getStringListParam("l");
                const state = params.getStringParam("s");
                if (!params.hasParam("s") && levels.length === 1) {
                    Log_1.Log.setLogLevel(Log_1.LogLevel[levels[0]], !Log_1.Log.getLogLevel(Log_1.LogLevel[levels[0]]));
                    result.push(`${(0, Log_1.logLevelToDisplayName)(Log_1.LogLevel[levels[0]]).padEnd(15, " ")}${Log_1.Log.getLogLevel(Log_1.LogLevel[levels[0]])}`);
                }
                else if (state === "true" || state === "false" || state === "t" || state === "f") {
                    for (const level of levels) {
                        Log_1.Log.setLogLevel(Log_1.LogLevel[level], state === "true" || state === "t");
                        result.push(`${(0, Log_1.logLevelToDisplayName)(Log_1.LogLevel[level]).padEnd(15, " ")}${state === "true" || state === "t"}`);
                    }
                }
                else {
                    result.push("-s must be true or false");
                }
            }
            else {
                result.push(`Current log levels`);
                result.push(``);
                // List log levels
                this.listLevels(result);
            }
            return result;
        };
        this.listLevels = (result) => {
            // List log levels
            const levels = Log_1.Log.getLogLevels();
            for (const level of Object.values(Log_1.LogLevel)) {
                result.push(`${(0, Log_1.logLevelToDisplayName)(level).padEnd(15, " ")}${levels[level] ?? false}`);
            }
        };
    }
}
exports.LogLevelCommand = LogLevelCommand;
