"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscribersCommand = void 0;
const CliCommand_1 = require("../CliCommand");
const SubscriberManager_1 = require("../../../subscribers/interfaces/SubscriberManager");
const lodash_1 = require("lodash");
const CliCommandDescription_1 = require("../CliCommandDescription");
class SubscribersCommand extends CliCommand_1.CliCommand {
    constructor() {
        super(CliCommandDescription_1.CliCommandDescription
            .make("subs", "Gets a list of subscribers and their connection status")
            .addFlag("d", "show details")
            .addAutoCompleteParameter("s", "service name", false, "", (soFar) => {
            const subscribers = SubscriberManager_1.SubscriberManager.get("subscriber").getAll();
            return (0, lodash_1.keys)(subscribers).filter(s => s.startsWith(soFar)).map(s => `${s},`);
        }));
        this.processCommand = async (params, _provider) => {
            let result = [];
            const detailed = params.hasParam("d");
            const services = params.getStringListParam("s");
            if (!detailed) {
                result = this.getBasicOutput(services?.length > 0 ? services : null);
            }
            else {
                result = this.getDetailedOutput(services?.length > 0 ? services : null);
            }
            return result;
        };
    }
    getBasicOutput(services) {
        const result = [];
        const subscribers = SubscriberManager_1.SubscriberManager.get("subscriber").getAll();
        result.push(`Connected  Service                   Provider                   Subscriber Type`);
        for (const service of (services ?? (0, lodash_1.keys)(subscribers))) {
            for (const type of (0, lodash_1.keys)(subscribers[service])) {
                const sub = subscribers[service][type].getSubscriber();
                const provider = sub.getProviderId() ?? "-";
                result.push(`${sub.isConnected()}`.padEnd(11, ' ') + `${service.padEnd(25, ' ')} ${provider.padEnd(26, " ")} ${type}`);
            }
        }
        return result;
    }
    getDetailedOutput(services) {
        const result = [];
        const subscribers = SubscriberManager_1.SubscriberManager.get("subscriber").getAll();
        for (const service of services || (0, lodash_1.keys)(subscribers)) {
            result.push("".padEnd(service.length, "="));
            result.push(`${service}`);
            for (const type of (0, lodash_1.keys)(subscribers[service])) {
                const sub = subscribers[service][type].getSubscriber();
                result.push("");
                result.push(`  ${type}`);
                result.push("  " + "".padEnd(type.length, "-"));
                const status = subscribers[service][type].getStatus();
                result.push(`  Created:             ${status.created}`);
                result.push(`  Connected:           ${status.connected}`);
                result.push(`  Protocol Version:    ${sub.getProtocolVersion()}`);
                result.push(`  Provider:            ${sub.getProviderId()}`);
                result.push(`  Last Connected:      ${status.connectedAt}`);
                result.push(`  Last Disconnected:   ${status.disconnectedAt}`);
                result.push(`  Connection Listeners:`);
                result.push(`   - ${"Connected:".padEnd(17, " ")} ${status.listeners.connected}`);
                result.push(`   - ${"Disconnected:".padEnd(17, " ")} ${status.listeners.disconnected}`);
                result.push(`  Status Messages:     ${status.statusMessages}`);
                result.push(`  Last Status Message: ${status.lastStatusAt}`);
                result.push(`  Status Keys:         ${sub.getStatusRoutingKeys().join(", ")}`);
                result.push(`  Status Listeners:`);
                result.push(`   - ${"*:".padEnd(36, " ")} ${status.listeners.status}`);
                for (const event of (0, lodash_1.keys)(status.listeners.statusMessages)) {
                    result.push(`   - ${(event + ":").padEnd(36, " ")} ${status.listeners.statusMessages[event]}`);
                }
            }
            result.push("");
        }
        return result;
    }
}
exports.SubscribersCommand = SubscribersCommand;
