"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DirectoryEntry = exports.ProviderEntry = void 0;
const ProviderState_1 = require("./ProviderState");
/**
 * Created by spencerrodd on 30/10/2015.
 */
const _ = require("lodash");
class ProviderEntry {
    constructor(providerId, providerState) {
        this.providerId = providerId;
        this.providerState = providerState;
        this.timeout = 2;
    }
    getId() {
        return this.providerId;
    }
    getState() {
        return this.providerState;
    }
    toString() {
        return "providerId=" + this.providerId + " providerState=" + this.providerState;
    }
    toJson() {
        return {
            providerId: this.providerId,
            providerState: this.providerState
        };
    }
    setProviderState(providerState) {
        this.providerState = providerState;
        this.timeout = 2;
    }
    decrementTimeout() {
        var result = false;
        if (this.timeout > 0) {
            this.timeout--;
            if (this.timeout === 0) {
                result = true;
                this.providerState = ProviderState_1.ProviderState.nocomms;
            }
        }
        return result;
    }
}
exports.ProviderEntry = ProviderEntry;
class DirectoryEntry {
    static makeFromJson(json) {
        return new DirectoryEntry(json.serviceId, json.providerId);
    }
    constructor(serviceId, providerId) {
        this.serviceId = serviceId;
        this.providers = [];
        for (var id of providerId) {
            this.providers.push(new ProviderEntry(id, ProviderState_1.ProviderState.nocomms));
        }
    }
    addProvider(providerId) {
        this.providers.push(new ProviderEntry(providerId, ProviderState_1.ProviderState.nocomms));
    }
    deleteProvider(providerId) {
        _.remove(this.providers, (providerEntry) => {
            return providerEntry.getId() === providerId;
        });
    }
    getServiceId() {
        return this.serviceId;
    }
    getAvailableProviderId() {
        for (var prov of this.providers) {
            if ((prov.getState() === ProviderState_1.ProviderState.active) || (prov.getState() === ProviderState_1.ProviderState.online)) {
                return prov.getId();
            }
        }
        return null;
    }
    getProviders() {
        return this.providers;
    }
    toString() {
        var result;
        result = "serviceId=" + this.serviceId + " providers=";
        for (var prov of this.providers) {
            result += "." + prov.toString() + ".";
        }
        return result;
    }
    providersAsJson() {
        var result = [];
        for (var prov of this.providers) {
            result.push(prov.toJson());
        }
        return result;
    }
    decrementTimeout() {
        var result = false;
        for (var prov of this.providers) {
            if (prov.decrementTimeout()) {
                result = true;
            }
        }
        return result;
    }
    toJson() {
        return { "serviceId": this.serviceId, "provider": this.providersAsJson() };
    }
}
exports.DirectoryEntry = DirectoryEntry;
