"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileTransferProvider = void 0;
const fs = require("fs-extra-promise");
const glob = require("glob");
const path = require("path");
class FileTransferProvider {
    constructor(providerId) {
        this.providerId = providerId;
    }
    async deleteFile(incoming) {
        const { file } = incoming.getParams();
        if (await fs.existsAsync(file)) {
            await fs.unlinkAsync(file);
        }
        return { status: "ok" };
    }
    getFilesInFolder(incoming) {
        const filePath = incoming.getParams().path || '/';
        const globPattern = incoming.getParams()["*"] || '*';
        const file = path.join(filePath, globPattern);
        return new Promise(((resolve, reject) => {
            glob(file, {}, (err, files) => {
                if (err) {
                    reject(err);
                }
                resolve(files);
            });
        }));
    }
    async getFileDetails(incoming) {
        const file = incoming.getParams().file;
        const stats = await fs.statAsync(file);
        stats.date = stats.mtime.toISOString();
        return stats;
    }
    async getFileBlock(incoming) {
        const file = incoming.getParams().file;
        const offset = incoming.getParams().offset;
        const blockSize = incoming.getParams().blockSize;
        const fd = await fs.openAsync(file, 'r');
        const buffer = Buffer.alloc(blockSize);
        return new Promise((resolve, reject) => {
            fs.read(fd, buffer, 0, blockSize, offset, (err, bytesRead, buff) => {
                if (err) {
                    reject(err);
                }
                const readBuff = Buffer.alloc(bytesRead);
                buff.copy(readBuff, 0, 0, bytesRead);
                fs.closeSync(fd);
                resolve({
                    blockSize: bytesRead,
                    block: readBuff.toJSON().data
                });
            });
        });
    }
    async writeFileBlock(incoming) {
        const file = incoming.getParams().file;
        const offset = incoming.getParams().offset;
        const block = incoming.getParams().block;
        if (!await fs.existsAsync(file)) {
            await fs.createFileAsync(file);
        }
        const fd = await fs.openAsync(file, 'r+');
        const buffer = Buffer.from(block);
        return new Promise((resolve, reject) => {
            fs.write(fd, buffer, 0, buffer.length, offset, (err, bytesWritten, buff) => {
                if (err) {
                    reject(err);
                }
                fs.closeSync(fd);
                resolve({
                    blockSize: bytesWritten
                });
            });
        });
    }
}
exports.FileTransferProvider = FileTransferProvider;
