"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Builder = exports.InfoBlockBuilder = exports.JSONSubTypeLookup = exports.InfoTypeLookup = void 0;
const _ = require("lodash");
// Add lookup between info type and info type block here, this will be filled out by individual info blocks
exports.InfoTypeLookup = {};
// Add lookup between info type and info type block here, this will be filled out by individual info blocks
exports.JSONSubTypeLookup = {};
class InfoBlockBuilder {
    constructor(jsonInfo, jobBuilder) {
        this.jsonInfo = jsonInfo;
        this.jobBuilder = jobBuilder;
        if (this.jobBuilder) {
            this.job = this.jobBuilder.get();
        }
        return this.fromJson(jsonInfo);
    }
    // public commit(keepVersion = false): AbstractInfoBlock {
    //     if (!keepVersion) {
    //         this.infoBlock.data.uuid = uuid.v4();
    //         this.infoBlock.data.addedAt = new Date();
    //         this.infoBlock.data.version = this.infoBlock.data.version ? this.infoBlock.data.version+1 : 1;
    //     }
    //     if (this.changeCallback) {
    //         this.changeCallback(this.infoBlock);
    //     }
    //     return this.infoBlock;
    //
    // }
    get() {
        return this.jsonInfo;
    }
    commit() {
        if (!this.job.info) {
            this.job.info = [];
        }
        this.job.info.push(this.jsonInfo);
        // if the userUuid has not been set in this builder, and it has been set in the jobBuilder
        // then update the userUuid from the jobBuilder
        if (!this.jsonInfo.userUuid && this.jobBuilder && this.jobBuilder.getUser()) {
            this.jsonInfo.userUuid = this.jobBuilder.getUser().uuid;
        }
        if (this.changeCallback) {
            this.changeCallback(this.jsonInfo);
        }
        return this.jobBuilder;
    }
    tap(tapFn) {
        if (tapFn) {
            tapFn(this.jsonInfo);
        }
        return this;
    }
    setUser(user) {
        this.jsonInfo.userUuid = user.uuid;
        return this;
    }
    fromJson(json) {
        this.jsonInfo = json;
        if (!this.jsonInfo.info) {
            this.jsonInfo.info = {};
        }
        return this;
    }
    toJson() {
        return this.jsonInfo.info;
    }
    onChange(callbackFn) {
        this.changeCallback = callbackFn;
        return this;
    }
    updateValue(newValue) {
        this.jsonInfo.info = _.extend({}, this.jsonInfo.info, newValue);
        return this;
    }
    async stateStoreMergeInfo() {
        if (this.jobBuilder && this.jobBuilder.stateStoreSubscriber) {
            await this.jobBuilder.stateStoreSubscriber.updateJobInfo({
                jobUuid: this.job.uuid,
                infoType: this.jsonInfo.infoType,
                subType: this.jsonInfo.subType,
                info: this.jsonInfo.info
            }, true);
        }
        else {
            throw Error("State store subscriber not defined");
        }
    }
}
exports.InfoBlockBuilder = InfoBlockBuilder;
exports.Builder = InfoBlockBuilder;
