"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EdlInfoBlock = void 0;
const _ = require("lodash");
const InfoBlockBuilder_1 = require("../common/InfoBlockBuilder");
class EdlInfoBlock extends InfoBlockBuilder_1.InfoBlockBuilder {
    constructor(jsonInfo, jobBuilder) {
        super({ ...jsonInfo, infoType: 'edl' }, jobBuilder);
    }
    addEdit(edit) {
        const newEdits = this.jsonInfo.info.edits || [];
        newEdits.push(edit || {});
        return this.updateValue({
            edits: newEdits
        });
    }
    setMaterialId(materialId) {
        const edit = _.last(this.jsonInfo.info.edits);
        if (edit) {
            edit.materialId = materialId;
        }
        return this;
    }
    setSom(som) {
        const edit = _.last(this.jsonInfo.info.edits);
        if (edit) {
            edit.som = som.getAsTcString();
        }
        return this;
    }
    setDuration(duration) {
        const edit = _.last(this.jsonInfo.info.edits);
        if (edit) {
            edit.duration = duration.getAsTcString();
        }
        return this;
    }
    setTimelinePosition(timelinePosition) {
        const edit = _.last(this.jsonInfo.info.edits);
        if (edit) {
            edit.timelinePosition = timelinePosition.getAsTcString();
        }
        return this;
    }
    setEdits(edits) {
        return this.updateValue({
            edits
        });
    }
}
exports.EdlInfoBlock = EdlInfoBlock;
if (!InfoBlockBuilder_1.InfoTypeLookup['edl']) {
    InfoBlockBuilder_1.InfoTypeLookup['edl'] = {};
}
InfoBlockBuilder_1.InfoTypeLookup['edl'].builder = EdlInfoBlock;
