"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestoreInfoBlock = void 0;
const _ = require("lodash");
const InfoBlockBuilder_1 = require("../common/InfoBlockBuilder");
const RESTORE_INFO_TYPE = 'restore';
class RestoreInfoBlock extends InfoBlockBuilder_1.InfoBlockBuilder {
    constructor(jsonInfo, jobBuilder) {
        super({ ...jsonInfo, infoType: RESTORE_INFO_TYPE }, jobBuilder);
    }
    addRestore(restore) {
        const newRestores = this.jsonInfo.info.restores || [];
        newRestores.push(restore || {});
        return this.updateValue({
            restores: newRestores
        });
    }
    setFormat(format) {
        return this.updateValue({ format });
    }
    setInputFileLocator(inputFileLocator) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.inputFileLocator = inputFileLocator;
        }
        return this;
    }
    setOutputFileLocator(outputFileLocator) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.outputFileLocator = outputFileLocator;
        }
        return this;
    }
    setMaterialId(materialId) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.materialId = materialId;
        }
        return this;
    }
    setSom(som) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.som = som.getAsTcString();
        }
        return this;
    }
    setDuration(duration) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.duration = duration.getAsTcString();
        }
        return this;
    }
    setChannel(channel) {
        const restore = _.last(this.jsonInfo.info.restores);
        if (restore) {
            restore.channel = channel;
        }
        return this;
    }
}
exports.RestoreInfoBlock = RestoreInfoBlock;
if (!InfoBlockBuilder_1.InfoTypeLookup['restore']) {
    InfoBlockBuilder_1.InfoTypeLookup['restore'] = {};
}
InfoBlockBuilder_1.InfoTypeLookup['restore'].builder = RestoreInfoBlock;
