"use strict";
/**
 * Created by spencerrodd on 08/11/2015.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Log = exports.logDisplayNameToLevel = exports.logLevelToFunction = exports.logLevelToDisplayName = exports.LogLevel = exports.DebugLibrary = void 0;
const _ = require("lodash");
const fs = require("fs");
const path = require("path");
const os = require("os");
const Debug = require("debug");
exports.DebugLibrary = Debug;
const util = require("util");
const JSZip = require("jszip");
const esbInterface_1 = require("../esbInterface");
const luxon_1 = require("luxon");
if (Debug?.inspectOpts) {
    Debug.inspectOpts.colors = true;
}
const debug = Debug("busby");
const defaultLog = Debug.log;
var LogLevel;
(function (LogLevel) {
    LogLevel["ERROR"] = "ERROR";
    LogLevel["WARN"] = "WARN";
    LogLevel["INFO"] = "INFO";
    LogLevel["DEBUG"] = "DEBUG";
    LogLevel["TRACE"] = "TRACE";
    LogLevel["API"] = "API";
    LogLevel["AUDIT"] = "AUDIT";
    LogLevel["STATUS"] = "STATUS";
    LogLevel["ESB"] = "ESB";
    LogLevel["EXCEPTION"] = "EXCEPTION";
    LogLevel["NAMESPACES"] = "NAMESPACES";
    LogLevel["NETWORK"] = "NETWORK";
    LogLevel["SCRIPT"] = "SCRIPT";
})(LogLevel || (exports.LogLevel = LogLevel = {}));
function logLevelToDisplayName(level) {
    switch (level) {
        case LogLevel.ERROR: return 'ERROR';
        case LogLevel.WARN: return 'WARN';
        case LogLevel.INFO: return 'INFO';
        case LogLevel.DEBUG: return 'DEBUG';
        case LogLevel.TRACE: return 'TRACE';
        case LogLevel.API: return 'API';
        case LogLevel.AUDIT: return 'AUDIT';
        case LogLevel.NETWORK: return 'NETWORK';
        case LogLevel.STATUS: return 'STATUS';
        case LogLevel.ESB: return 'ESB';
        case LogLevel.EXCEPTION: return 'EXCEPTION';
        case LogLevel.NAMESPACES: return 'NAMESPACES';
        case LogLevel.SCRIPT: return 'SCRIPT';
        default: return '';
    }
}
exports.logLevelToDisplayName = logLevelToDisplayName;
function logLevelToFunction(level, subFile) {
    subFile ??= "__default";
    switch (level) {
        case LogLevel.ERROR: return Log.getSubFile(subFile).logError;
        case LogLevel.WARN: return Log.getSubFile(subFile).logWarn;
        case LogLevel.INFO: return Log.getSubFile(subFile).logInfo;
        case LogLevel.DEBUG: return Log.getSubFile(subFile).logDebug;
        case LogLevel.TRACE: return Log.getSubFile(subFile).logTrace;
        case LogLevel.API: return Log.getSubFile(subFile).logAPI;
        case LogLevel.AUDIT: return Log.getSubFile(subFile).logAudit;
        case LogLevel.NETWORK: return Log.getSubFile(subFile).logNetwork;
        case LogLevel.STATUS: return Log.getSubFile(subFile).logStatus;
        case LogLevel.ESB: return Log.getSubFile(subFile).logESB;
        case LogLevel.SCRIPT: return Log.getSubFile(subFile).logScript;
        // case LogLevel.EXCEPTION: return Log.logException;
        default: return () => { };
    }
}
exports.logLevelToFunction = logLevelToFunction;
function logDisplayNameToLevel(level) {
    switch (level.toLowerCase()) {
        case 'error': return LogLevel.ERROR;
        case 'warn': return LogLevel.WARN;
        case 'info': return LogLevel.INFO;
        case 'debug': return LogLevel.DEBUG;
        case 'trace': return LogLevel.TRACE;
        case 'api': return LogLevel.API;
        case 'network': return LogLevel.NETWORK;
        case 'audit': return LogLevel.AUDIT;
        case 'status': return LogLevel.STATUS;
        case 'script': return LogLevel.SCRIPT;
        case 'esb': return LogLevel.ESB;
        case 'exeception': return LogLevel.EXCEPTION;
        case 'namespaces': return LogLevel.NAMESPACES;
        default: throw new Error(`log level=${level} not recognised`);
    }
}
exports.logDisplayNameToLevel = logDisplayNameToLevel;
class LogEvent {
    constructor(logLevel, message, subfile) {
        this.logLevel = logLevel;
        this.message = message;
        this.subfile = subfile;
        this.date = luxon_1.DateTime.utc();
    }
    getLogLevel() {
        return this.logLevel;
    }
    getMessage() {
        return this.message;
    }
    getDate() {
        return this.date;
    }
    getSubfile() {
        return this.subfile;
    }
    toString() {
        return this.date.toISO() +
            " " + LogLevel[this.logLevel] +
            ":" + this.message;
    }
}
process.on("uncaughtException", (err) => {
    Log.logException(err);
    Log.logError("Shutdown Exception raised: " + err.message);
    // @ts-ignore // Using custom event which doesn't exist on process
    process.emit("shutdownException", err);
});
process.on('unhandledRejection', (err, p) => {
    Log.logError(`Unhandled Promise Rejection - ${err}\n${err?.stack ?? ""}`, err);
});
process.on('SIGUSR2', () => {
    Log.SIGUSR2Handler();
});
class Log {
    constructor(subfile) {
        this.subfile = subfile;
        this.logError = (errorMessage, parentErrorMessage) => {
            Log.getNamespace("log:error" + (this.subfile ? ":" + this.subfile : ""))(errorMessage, parentErrorMessage ?? "");
            this.logToConsole(LogLevel.ERROR, errorMessage);
            if (parentErrorMessage) {
                this.logToConsole(LogLevel.ERROR, parentErrorMessage.message);
                errorMessage = errorMessage + "\n-- " + parentErrorMessage.message + "\n-- " + parentErrorMessage.stack;
            }
            if (Log.checkLevel(LogLevel.ERROR)) {
                Log.runTimer(new LogEvent(LogLevel.ERROR, errorMessage, this.subfile));
            }
        };
        this.logEx = (message, err) => {
            this.logToConsole(LogLevel.EXCEPTION, message + " " + err.message);
            this.logToConsole(LogLevel.EXCEPTION, err.stack);
            const errorMessage = message + " " + err.message + "\n-- " + err.stack;
            Log.runTimer(new LogEvent(LogLevel.EXCEPTION, errorMessage, this.subfile));
        };
        this.logException = (err) => {
            this.logToConsole(LogLevel.EXCEPTION, err.message);
            this.logToConsole(LogLevel.EXCEPTION, err.stack);
            const errorMessage = err.message + "\n-- " + err.stack;
            Log.runTimer(new LogEvent(LogLevel.EXCEPTION, errorMessage, this.subfile));
        };
        this.logErrorAndReject = (errorMessage, parentErrorMessage) => {
            let error;
            if (errorMessage instanceof Error) {
                error = errorMessage;
            }
            else {
                error = new Error(errorMessage);
            }
            this.logError(error.message, parentErrorMessage);
            return Promise.reject(error);
        };
        this.logErrorAndThrow = (errorMessage, parentErrorMessage) => {
            let error;
            if (errorMessage instanceof Error) {
                error = errorMessage;
            }
            else {
                error = new Error(errorMessage);
            }
            this.logError(error.message, parentErrorMessage);
            throw (error);
        };
        this.logErrorAndExit = (errorMessage, parentErrorMessage, exitCode) => {
            let error;
            if (errorMessage instanceof Error) {
                error = errorMessage;
            }
            else {
                error = new Error(errorMessage);
            }
            this.logError(error.message, parentErrorMessage);
            Log.flushAndExit(exitCode);
        };
        this.logToConsole = (logLevel, msg) => {
            if (process.env.NODE_ENV === 'development' && !Debug.enabled("busby:log")) {
                let formattedMessage;
                if (this.subfile) {
                    formattedMessage = luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS') + " " + LogLevel[logLevel] + " " + (Log.filename || '') + '(' + this.subfile + '): ' + msg;
                }
                else {
                    formattedMessage = luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS') + " " + LogLevel[logLevel] + " " + (Log.filename || '') + ': ' + msg;
                }
                if (logLevel === LogLevel.ERROR) {
                    console.error(formattedMessage);
                }
                else {
                    console.log(formattedMessage);
                }
            }
        };
        this.logInfo = (infoMessage) => {
            Log.getNamespace("log:info" + (this.subfile ? ":" + this.subfile : ""))(infoMessage);
            this.logToConsole(LogLevel.INFO, infoMessage);
            if (Log.checkLevel(LogLevel.INFO)) {
                Log.runTimer(new LogEvent(LogLevel.INFO, infoMessage, this.subfile));
            }
        };
        this.logAudit = (auditMessage) => {
            Log.getNamespace("log:audit" + (this.subfile ? ":" + this.subfile : ""))(auditMessage);
            this.logToConsole(LogLevel.AUDIT, auditMessage);
            if (Log.checkLevel(LogLevel.AUDIT)) {
                Log.runTimer(new LogEvent(LogLevel.AUDIT, auditMessage, this.subfile));
            }
        };
        this.logNetwork = (networkMessage) => {
            Log.getNamespace("log:network" + (this.subfile ? ":" + this.subfile : ""))(networkMessage);
            if (Log.checkLevel(LogLevel.NETWORK)) {
                this.logToConsole(LogLevel.NETWORK, networkMessage);
                Log.runTimer(new LogEvent(LogLevel.NETWORK, networkMessage, this.subfile));
            }
        };
        this.logStatus = (statusMessage) => {
            Log.getNamespace("log:status" + (this.subfile ? ":" + this.subfile : ""))(statusMessage);
            if (Log.checkLevel(LogLevel.STATUS)) {
                this.logToConsole(LogLevel.STATUS, statusMessage);
                Log.runTimer(new LogEvent(LogLevel.STATUS, statusMessage, this.subfile));
            }
        };
        this.logScript = (scriptMessage) => {
            Log.getNamespace("log:script" + (this.subfile ? ":" + this.subfile : ""))(scriptMessage);
            this.logToConsole(LogLevel.SCRIPT, scriptMessage);
            if (Log.checkLevel(LogLevel.SCRIPT)) {
                Log.runTimer(new LogEvent(LogLevel.SCRIPT, scriptMessage, this.subfile));
            }
        };
        this.logDebug = (debugMessage) => {
            Log.getNamespace("log:debug" + (this.subfile ? ":" + this.subfile : ""))(debugMessage);
            this.logToConsole(LogLevel.DEBUG, debugMessage);
            if (Log.checkLevel(LogLevel.DEBUG)) {
                Log.runTimer(new LogEvent(LogLevel.DEBUG, debugMessage, this.subfile));
            }
        };
        this.logWarn = (warnMessage) => {
            Log.getNamespace("log:warn" + (this.subfile ? ":" + this.subfile : ""))(warnMessage);
            this.logToConsole(LogLevel.WARN, warnMessage);
            if (Log.checkLevel(LogLevel.WARN)) {
                Log.runTimer(new LogEvent(LogLevel.WARN, warnMessage, this.subfile));
            }
        };
        this.logAPI = (apiMessage) => {
            Log.getNamespace("log:api" + (this.subfile ? ":" + this.subfile : ""))(apiMessage);
            if (Log.checkLevel(LogLevel.API)) {
                this.logToConsole(LogLevel.API, apiMessage);
                Log.runTimer(new LogEvent(LogLevel.API, apiMessage, this.subfile));
            }
        };
        this.logTrace = (traceMessage) => {
            Log.getNamespace("log:trace" + (this.subfile ? ":" + this.subfile : ""))(traceMessage);
            if (Log.checkLevel(LogLevel.TRACE)) {
                this.logToConsole(LogLevel.TRACE, traceMessage);
                Log.runTimer(new LogEvent(LogLevel.TRACE, traceMessage, this.subfile));
            }
        };
        this.logESB = (esbMessage) => {
            if (Log.checkLevel(LogLevel.ESB)) {
                if (esbMessage instanceof esbInterface_1.EsbMessage) {
                    this.logToConsole(LogLevel.ESB, JSON.stringify(esbMessage));
                    Log.runTimer(new LogEvent(LogLevel.ESB, JSON.stringify(esbMessage), this.subfile));
                }
                else {
                    this.logToConsole(LogLevel.ESB, esbMessage);
                    Log.runTimer(new LogEvent(LogLevel.ESB, esbMessage, this.subfile));
                }
            }
        };
    }
    static disableFileLogging() {
        Log.disableLogFile = true;
    }
    static addLogDestination(destination) {
        Log.logDestinations.add(destination);
        return {
            dispose: () => this.removeLogDestination(destination),
        };
    }
    static removeLogDestination(destination) {
        Log.logDestinations.delete(destination);
    }
    static SIGUSR2Handler(enable) {
        if (enable === false || Log.fullLogging) {
            Log.logInfo(`Disabling full logging`);
            Log.fullLogging = false;
        }
        else {
            Log.logInfo(`Enabling full logging`);
            Log.fullLogging = true;
        }
    }
    static checkLevel(logLevel) {
        let result;
        result = Log.fullLogging;
        if (!Log.fullLogging && Log.toLog.hasOwnProperty([logLevel].toString())) {
            result = Log.toLog[logLevel];
        }
        return result;
    }
    static isNode() {
        const checkNode = new Function("try {return this===global;}catch(e){return false;}");
        return checkNode();
    }
    static setFileName(filename) {
        if (!fs.existsSync(Log.logPath)) {
            fs.mkdirSync(Log.logPath, { recursive: true });
        }
        Log.filename = filename;
        Log.logInfo("**** Application " + this.filename + " started ****");
        debug.log = (format, ...args) => {
            this.runTimer(new LogEvent(LogLevel.NAMESPACES, util.format(format, ...args), "debug"));
            if (!process?.env?.BUSBY_MONITOR) {
                defaultLog(format, ...args);
            }
        };
    }
    static startLogging() {
        if (!fs.existsSync(Log.logPath)) {
            fs.mkdirSync(Log.logPath, { recursive: true });
        }
        Log.logInfo("**** Application " + this.filename + " started ****");
        debug.log = (format, ...args) => {
            this.runTimer(new LogEvent(LogLevel.NAMESPACES, util.format(format, ...args), "debug"));
            if (!process?.env?.BUSBY_MONITOR) {
                defaultLog(format, ...args);
            }
        };
    }
    static getSubFile(subFile) {
        return Log.logFiles[subFile];
    }
    static getFileName() {
        return Log.filename;
    }
    static setKeepDays(keep) {
        Log.keepDays = keep;
    }
    static setZipAfter(zipAfter) {
        Log.zipAfter = zipAfter;
    }
    static getKeepDays() {
        return Log.keepDays;
    }
    static setLogLevel(logLevel, onOff) {
        Log.logInfo(`Setting log level ${logLevelToDisplayName(logLevel) || `[!${logLevel}]`} to ${onOff}`);
        Log.toLog[logLevel] = onOff;
    }
    static getLogLevel(logLevel) {
        if (Log.toLog.hasOwnProperty([logLevel].toString())) {
            return Log.toLog[logLevel];
        }
        else {
            return false;
        }
    }
    static getLogLevels() {
        Log.logInfo(JSON.stringify(this.toLog));
        return { ...this.toLog };
    }
    static logError(errorMessage, parentErrorMessage) {
        Log.logFiles.__default.logError(errorMessage, parentErrorMessage);
    }
    static logEx(message, err) {
        Log.logFiles.__default.logEx(message, err);
    }
    static logException(err) {
        Log.logFiles.__default.logException(err);
    }
    static logErrorAndReject(errorMessage, parentErrorMessage) {
        return Log.logFiles.__default.logErrorAndReject(errorMessage, parentErrorMessage);
    }
    static logErrorAndThrow(errorMessage, parentErrorMessage) {
        Log.logFiles.__default.logErrorAndThrow(errorMessage, parentErrorMessage);
    }
    static logErrorAndExit(errorMessage, parentErrorMessage, exitCode) {
        Log.logFiles.__default.logErrorAndExit(errorMessage, parentErrorMessage, exitCode);
    }
    static logInfo(infoMessage) {
        Log.logFiles.__default.logInfo(infoMessage);
    }
    static logAudit(auditMessage) {
        Log.logFiles.__default.logAudit(auditMessage);
    }
    static logNetwork(networkMessage) {
        Log.logFiles.__default.logNetwork(networkMessage);
    }
    static logStatus(statusMessage) {
        Log.logFiles.__default.logStatus(statusMessage);
    }
    static logScript(scriptMessage) {
        Log.logFiles.__default.logScript(scriptMessage);
    }
    static logDebug(debugMessage) {
        Log.logFiles.__default.logDebug(debugMessage);
    }
    static logWarn(warnMessage) {
        Log.logFiles.__default.logWarn(warnMessage);
    }
    static logAPI(apiMessage) {
        Log.logFiles.__default.logAPI(apiMessage);
    }
    static logTrace(traceMessage) {
        Log.logFiles.__default.logTrace(traceMessage);
    }
    static logESB(esbMessage) {
        Log.logFiles.__default.logESB(esbMessage);
    }
    /**
     * Gets a debug namespace which can be enabled using `DEBUG` environment variable or localstorage `debug` in the browser.
     * @param namespace a colon seperated string which will add a debug namespace of `busby:{namespace}`
     */
    static getNamespace(namespace) {
        if (!Log.namespaces[namespace]) {
            Log.namespaces[namespace] = debug.extend(namespace);
        }
        return Log.namespaces[namespace];
    }
    static flushAndExit(exitCode = 1) {
        this.exiting = true;
        this.exitCode = exitCode;
        this.runTimer(new LogEvent(LogLevel.WARN, `Exiting with exit code ${exitCode}`));
    }
    static runTimer(logEvent) {
        if (logEvent) {
            Log.logEventList.unshift(logEvent);
        }
        if (Log.interval === null) {
            const logNow = () => {
                let count = 0;
                let promise;
                while ((Log.logEventList.length > 0) && (count < 20)) {
                    const le = Log.logEventList.pop();
                    let cb;
                    if (this.exiting) {
                        promise = new Promise((resolve) => cb = resolve);
                    }
                    if (!this.disableLogFile) {
                        Log.writeToFile(le, cb);
                    }
                    try {
                        for (const destination of this.logDestinations.values()) {
                            destination(le);
                        }
                    }
                    catch { }
                    count++;
                }
                if (Log.logEventList.length === 0) {
                    clearTimeout(Log.interval);
                    Log.interval = null;
                    if (this.exiting) {
                        if (promise) {
                            promise.then(() => {
                                process.exit(this.exitCode);
                            });
                        }
                        else {
                            process.exit(this.exitCode);
                        }
                    }
                }
                else {
                    Log.interval = setTimeout(logNow, 10);
                }
            };
            Log.interval = setTimeout(logNow, 10);
        }
    }
    static writeToFile(logEvent, cb) {
        if (this.isNode()) {
            if (!Log.initialTidyDone) {
                Log.logInfo("Zipping and deleting old log files");
                Log.zipAndTidyOldLogFiles();
                Log.initialTidyDone = true;
            }
            if (typeof process !== 'undefined') {
                fs.appendFile(Log.getCurrentFilename(logEvent.getDate(), logEvent.getSubfile()), logEvent.toString() + "\n", (err) => {
                    if (err) {
                        Log.logError("error writing to log " + err);
                        console.error("error writing to log " + err);
                    }
                    cb?.();
                });
            }
        }
    }
    static getCurrentFilename(eventDate, subfile) {
        if (eventDate.startOf("day") > Log.currentDate.startOf("day")) {
            Log.logInfo("Date change zipping and deleting old log files " + Log.currentDate.startOf("day") + "+" + eventDate.startOf("day"));
            Log.zipAndTidyOldLogFiles();
            Log.currentDate = eventDate;
        }
        if (subfile) {
            return Log.logPath + path.sep + Log.filename + "_" + subfile + "_" + Log.shortDate(eventDate) + ".log";
        }
        else {
            return Log.logPath + path.sep + Log.filename + "_" + Log.shortDate(eventDate) + ".log";
        }
    }
    static zipAndTidyOldLogFiles() {
        if (this.isNode()) { //are we in node rather than the browser?
            fs.readdir(Log.logPath, (err, files) => {
                if (err) {
                    Log.logError("Error tidying old log files: " + err);
                }
                else {
                    for (const file of files) {
                        if (file.indexOf(Log.filename) === 0) {
                            const fileToCheck = Log.logPath + path.sep + file;
                            fs.stat(fileToCheck, (err, stats) => {
                                if (!err) {
                                    // Check if file older than maximum keep days
                                    if (Log.currentDate.valueOf() - stats.mtime.valueOf() > Log.keepDays * (24 * 60 * 60 * 1000)) {
                                        Log.logInfo("Deleting old log file " + fileToCheck);
                                        fs.unlink(fileToCheck, (err) => {
                                            if (err) {
                                                Log.logError("Error deleting log file " + fileToCheck);
                                            }
                                        });
                                    }
                                    else if (!file.endsWith(".zip") && Log.currentDate.valueOf() - stats.mtime.valueOf() > Log.zipAfter * (24 * 60 * 60 * 1000)) {
                                        Log.logInfo(`Zipping old log file ${fileToCheck}`);
                                        const outputZip = fileToCheck + ".zip";
                                        try {
                                            this.zipFile(fileToCheck, outputZip);
                                        }
                                        catch (err) {
                                            Log.logError("Error zipping up file " + fileToCheck + " - " + err);
                                        }
                                    }
                                }
                            });
                        }
                    }
                }
            });
        }
    }
    static zipFile(fileToZip, outputFile) {
        const zip = new JSZip();
        zip.file(path.basename(fileToZip), fs.createReadStream(fileToZip));
        zip.generateNodeStream({ type: 'nodebuffer', streamFiles: true }).
            pipe(fs.createWriteStream(outputFile)).
            on('finish', () => {
            fs.unlink(fileToZip, (err) => {
                if (err) {
                    Log.logError("cannot delete log file " + fileToZip);
                }
                else {
                    Log.logInfo("zipped log file " + fileToZip);
                }
            });
        });
    }
    static shortDate(dt) {
        return dt.toFormat("yyyyMMdd");
    }
}
exports.Log = Log;
Log.logFiles = new Proxy({}, {
    get(target, subFile, receiver) {
        target[subFile] ??= new Log(subFile === "__default" ? undefined : subFile);
        return target[subFile];
    },
});
Log.disableLogFile = false;
Log.logEventList = [];
Log.filename = _.get(process, ['argv', 5]);
Log.interval = null;
Log.currentDate = luxon_1.DateTime.utc();
Log.keepDays = 365;
Log.zipAfter = 7;
Log.logPath = (process.env.NODE_ENV === 'development' || process.env.USER_MODE === "local") ? path.join(os.homedir(), "log", "squaredpaper") : "/var/log/squaredpaper";
Log.initialTidyDone = false;
Log.exiting = false;
Log.exitCode = -1;
Log.logDestinations = new Set();
Log.toLog = {
    [LogLevel.ERROR]: true,
    [LogLevel.WARN]: true,
    [LogLevel.INFO]: true
};
Log.fullLogging = false;
Log.namespaces = {};
