"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NeptuneNormaliser = void 0;
const bluebird_1 = require("bluebird");
const _ = require("lodash");
const BDateTime_1 = require("../bTime/BDateTime");
const BDuration_1 = require("../bTime/BDuration");
const normaliser_1 = require("./normaliser");
const schedule_1 = require("../schedule");
const xml2js_1 = require("xml2js");
const ScheduleInfoBlock_1 = require("../infoBlocks/schedule/ScheduleInfoBlock");
const parseStringPromise = (0, bluebird_1.promisify)(xml2js_1.parseString);
class NeptuneNormaliser extends normaliser_1.Normaliser {
    getConfigForm() {
        return {
            schema: {
                "type": "object",
                "properties": {}
            }
        };
    }
    getDescription() {
        return "Reads info from a Neptune XML schedule";
    }
    getId() {
        return "neptune";
    }
    addMediaEvent(neptuneEvent, scheduleBuilder) {
        const startMode = neptuneEvent.starttype[0] === "SEQ" ? "follow" : "fixed";
        const duration = neptuneEvent.duration ? BDuration_1.BDuration.fromTcString(neptuneEvent.duration[0], "ndf25") : undefined;
        switch (startMode) {
            case "fixed":
                {
                    const [day, month, year] = neptuneEvent.onairdate[0].split(" ");
                    const startTime = BDateTime_1.BDateTime.fromJson({
                        time: neptuneEvent.onairtime[0],
                        date: `20${year}-${month}-${day}`,
                        frameRate: "ndf25"
                    });
                    scheduleBuilder.fixedEvent(startTime, { duration });
                }
                break;
            case "follow":
                {
                    scheduleBuilder.followEvent({ duration });
                }
                break;
        }
        if (neptuneEvent.mediaid) {
            const materialId = neptuneEvent.mediaid[0];
            const som = neptuneEvent.som ? BDateTime_1.BDateTime.fromTimecode(neptuneEvent.som[0], "ndf25") : null;
            scheduleBuilder.mediaSource(materialId, { som });
        }
        scheduleBuilder.reconcileId(neptuneEvent.reconcilekey?.[0]);
        scheduleBuilder.eventCategory(neptuneEvent.category?.[0]);
        scheduleBuilder.title(neptuneEvent.title[0]);
    }
    buildEventList(neptuneEvents, scheduleBuilder) {
        return _.map(neptuneEvents, (neptuneEvent) => {
            switch (neptuneEvent.$.type) {
                case "MEDIA":
                    {
                        this.addMediaEvent(neptuneEvent, scheduleBuilder);
                    }
                    break;
                case "COMMENT":
                    {
                    }
                    break;
            }
        });
    }
    buildSchedule(schedule, neptune) {
        const scheduleBuilder = new ScheduleInfoBlock_1.ScheduleInfoBlock(schedule);
        this.buildEventList(neptune.playlist.eventlist[0].event, scheduleBuilder);
        return scheduleBuilder.getSchedule();
    }
    async internalTranslate(data, options, incomingSchedule) {
        const neptune = await parseStringPromise(data);
        let schedule;
        if (incomingSchedule) {
            schedule = incomingSchedule;
        }
        else {
            schedule = new schedule_1.Schedule("channel", [], "name", "schedule");
        }
        return this.buildSchedule(schedule, neptune);
    }
}
exports.NeptuneNormaliser = NeptuneNormaliser;
// async function test() {
//     const normaliser = new NeptuneNormaliser();
//     const data = await fs.readFile("../../resources/neptuneSchedule.xml", "utf-8");
//     const schedule: NeptuneSchedule = await normaliser.translate(data, {});
//
//     console.log(schedule);
// }
//
// test();
