"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.xmlNormaliser = void 0;
const normaliser_1 = require("./normaliser");
const _ = require("lodash");
const xml2js_1 = require("xml2js");
class xmlNormaliser extends normaliser_1.Normaliser {
    getId() {
        return "xml";
    }
    getConfigForm() {
        return {
            schema: {
                type: "object",
                title: "Options",
                properties: {
                    includeElements: {
                        type: "array",
                        title: "Include elements",
                        items: {
                            type: "string"
                        }
                    },
                    objectArrayPath: {
                        type: "string",
                        title: "Object array path",
                    },
                    arrayNodes: {
                        type: "array",
                        title: "Array nodes",
                        default: [],
                        items: {
                            type: "string"
                        }
                    }
                }
            },
            uiSchema: {
                includeElements: {
                    "ui:options": {
                        orderable: false
                    }
                },
                arrayNodes: {
                    "ui:options": {
                        orderable: false
                    }
                }
            }
        };
    }
    getAsBuffer() {
        return false;
    }
    getDescription() {
        return "Read xml files";
    }
    internalTranslate(data, options) {
        return new Promise((resolve, reject) => {
            var parserOptions = { explicitArray: false };
            var xmlJson;
            (0, xml2js_1.parseString)(data, parserOptions, (err, result) => {
                xmlJson = (err) ? null : result;
                if (xmlJson) {
                    //Ensure arrays
                    const jsonString = JSON.stringify(xmlJson, (key, value) => {
                        if (_.includes(options.arrayNodes, key) && !_.isArray(value)) {
                            return [value];
                        }
                        else {
                            return value;
                        }
                    });
                    xmlJson = JSON.parse(jsonString);
                    var filteredJson = (options.objectArrayPath) ? _.get(xmlJson, options.objectArrayPath) : xmlJson;
                    if (options.includeElements) {
                        _.each(options.includeElements, (elementPath) => {
                            const value = _.get(xmlJson, elementPath);
                            const key = elementPath.split('.').pop();
                            filteredJson = _.map(filteredJson, (o) => _.merge(o, { [key]: value }));
                        });
                    }
                }
                return resolve(filteredJson);
            });
        });
    }
}
exports.xmlNormaliser = xmlNormaliser;
