"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InfoBlockPluginClient = void 0;
const logging_1 = require("../logging");
const BasePluginClient_1 = require("./BasePluginClient");
const debug = logging_1.Log.getNamespace("infoBlockPluginClient");
class InfoBlockPluginClient extends BasePluginClient_1.BasePluginClient {
    static get() {
        if (!InfoBlockPluginClient.__instance) {
            new InfoBlockPluginClient();
        }
        return InfoBlockPluginClient.__instance;
    }
    constructor() {
        super();
        this.onInfoBlockCallbacks = new Set();
        this.onInfoBlock = (callback) => {
            this.onInfoBlockCallbacks.add(callback);
            if (this.infoBlock) {
                callback(this.infoBlock, this.job, this.infoBlockConfig);
            }
            return { dispose: () => this.offInfoBlock(callback) };
        };
        this.offInfoBlock = (callback) => {
            this.onInfoBlockCallbacks.delete(callback);
        };
        InfoBlockPluginClient.__instance = this;
        window.__INFOBLOCKPLUGINCLIENT__ = this;
    }
    onPortMessage(msg) {
        super.onPortMessage(msg);
        switch (msg.data.type) {
            case "infoBlock/data": {
                debug("Received Info Block and Job", msg.data.infoBlock.uuid, msg.data.job.uuid);
                this.infoBlock = msg.data.infoBlock;
                this.job = msg.data.job;
                this.infoBlockConfig = msg.data.config;
                for (const callback of this.onInfoBlockCallbacks.values()) {
                    try {
                        callback(this.infoBlock, this.job, this.infoBlockConfig);
                    }
                    catch { }
                }
                break;
            }
        }
    }
    getJob() {
        return this.job;
    }
    getInfoBlockConfig() {
        return this.infoBlockConfig;
    }
    getInfoBlock() {
        return this.infoBlock;
    }
}
exports.InfoBlockPluginClient = InfoBlockPluginClient;
