"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginClient = exports.AppPluginClient = void 0;
const logging_1 = require("../logging");
const selector_1 = require("../selector");
const BasePluginClient_1 = require("./BasePluginClient");
const debug = logging_1.Log.getNamespace("appPluginClient");
class AppPluginClient extends BasePluginClient_1.BasePluginClient {
    static get() {
        if (!AppPluginClient.__instance) {
            new AppPluginClient();
        }
        return AppPluginClient.__instance;
    }
    constructor() {
        super();
        this.appSelectedCallbacks = new Set();
        this.appFocusedCallbacks = new Set();
        this.onUserMetadataCallbacks = new Set();
        this.pluginConnected = () => {
            selector_1.InterfaceUser.get().onUserMetadata(this.onAppMetadata);
        };
        this.onAppMetadata = (key, value, oldValue) => {
            if (key.startsWith(this.id)) {
                this.onUserMetadataCallbacks.forEach((fn) => {
                    try {
                        fn?.(key.replace(`${this.id}.`, ''), value, oldValue);
                    }
                    catch { }
                });
            }
        };
        /**
         * Tell selector to select your app.
         * This should be used sparingly, and usually as a result of some other trigger.
         */
        this.selectThisApp = (data) => {
            this.postMessage({
                type: "operation/selectThisApp",
                params: {
                    data,
                },
            });
        };
        this.onAppSelected = (callback) => {
            this.appSelectedCallbacks.add(callback);
            return {
                dispose: () => {
                    this.offAppSelected(callback);
                }
            };
        };
        this.offAppSelected = (callback) => {
            this.appSelectedCallbacks.delete(callback);
        };
        this.onAppFocused = (callback) => {
            this.appFocusedCallbacks.add(callback);
            return {
                dispose: () => {
                    this.offAppFocused(callback);
                }
            };
        };
        this.offAppFocused = (callback) => {
            this.appFocusedCallbacks.delete(callback);
        };
        this.getFocusedData = () => {
            return this.focusedData;
        };
        this.getSelectedData = () => {
            return this.selectedData;
        };
        AppPluginClient.__instance = this;
        window.__PLUGINCLIENT__ = this;
        this.onConnected(this.pluginConnected);
    }
    setUserAppMetadata(key, value) {
        this.postMessage({
            type: "userMetadata/app",
            params: {
                key,
                value,
            }
        });
    }
    getUserAppMetadata(key) {
        return this.getUserMetadata(`${this.id}.${key}`);
    }
    onAppUserMetadata(eventHandler) {
        this.onUserMetadataCallbacks.add(eventHandler);
        return {
            dispose: () => {
                this.offAppUserMetadata(eventHandler);
            }
        };
    }
    offAppUserMetadata(eventHandler) {
        this.onUserMetadataCallbacks.delete(eventHandler);
    }
    onPortMessage(msg) {
        super.onPortMessage(msg);
        switch (msg.data.type) {
            case "selected": {
                this.selectedData = msg.data.data;
                this.appSelectedCallbacks.forEach((fn) => {
                    try {
                        fn?.(msg.data.data);
                    }
                    catch { }
                });
                break;
            }
            case "focused": {
                this.focusedData = msg.data.data;
                this.appFocusedCallbacks.forEach((fn) => {
                    try {
                        fn?.(msg.data.data);
                    }
                    catch { }
                });
                break;
            }
        }
    }
    /**
     * Sets the alerting state for the app.
     * This shows the app in red in the app toolbar.
     * It will also make the selector favicon red until cleared.
     * @param alerting Whether the app should show an alert
     */
    setAppAlert(alerting) {
        this.postMessage({
            type: "operation/setAppAlert",
            params: {
                alerting,
            },
        });
    }
}
exports.AppPluginClient = AppPluginClient;
exports.PluginClient = AppPluginClient;
