"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayEvent = exports.BreakHeaderEvent = exports.CommentEvent = exports.PrimaryEvent = void 0;
const _ = require("lodash");
const transition_1 = require("./transition");
const source_1 = require("./source");
const secondaryEvents_1 = require("./secondaryEvents");
const bTime_1 = require("../bTime");
const subtitle_1 = require("./subtitle");
const audioTechnical_1 = require("./audioTechnical");
const asRun_1 = require("./asRun");
const OptOut_1 = require("./OptOut");
const audioDescription_1 = require("./audioDescription");
const blankingEvent_1 = require("./blankingEvent");
const gpoEvent_1 = require("./secondaryEvents/gpoEvent");
const dveEvent_1 = require("./secondaryEvents/dveEvent");
class PrimaryEvent {
    constructor(id, eventType, options) {
        this.id = id;
        this.eventType = eventType;
        if (!id) {
            throw new Error("Primary event missing ID");
        }
        if (options) {
            this.enabled = _.has(options, 'enabled') ? options.enabled : true;
            this.hold = _.has(options, 'hold') ? options.hold : false;
            this.title = options.title;
            this.custom = options.custom;
            this.customArray = options.customArray;
            this.passThrough = options.passThrough;
            this.comment = options.comment;
            this.previousEventId = options.previousEventId;
            this.previousReconcileId = options.previousReconcileId;
            this.groupId = options.groupId;
            this.groupName = options.groupName;
            this.groupType = options.groupType;
            this.action = options.action;
            this.scheduleOrigin = options.scheduleOrigin;
        }
    }
    setPreviousEventId(previousEventId) {
        this.previousEventId = previousEventId;
    }
    setPreviousReconcileId(previousReconcileId) {
        this.previousReconcileId = previousReconcileId;
    }
    toJson() {
        return {
            id: this.id,
            enabled: _.has(this, 'enabled') ? this.enabled : true,
            hold: _.has(this, 'hold') ? this.hold : true,
            title: this.title,
            eventType: this.eventType,
            custom: this.custom,
            customArray: this.customArray,
            passThrough: this.passThrough,
            comment: this.comment,
            previousEventId: this.previousEventId,
            previousReconcileId: this.previousReconcileId,
            groupId: this.groupId,
            groupName: this.groupName,
            groupType: this.groupType,
            action: this.action,
            scheduleOrigin: this.scheduleOrigin
        };
    }
}
exports.PrimaryEvent = PrimaryEvent;
class CommentEvent extends PrimaryEvent {
    constructor(id, comment, options) {
        super(id, 'comment', options && {
            title: options.title,
            custom: options.custom,
            groupId: options.groupId,
            groupName: options.groupName,
            groupType: options.groupType,
            previousEventId: options.previousEventId,
            scheduleOrigin: options.scheduleOrigin,
            action: options.action
        });
        this.comment = comment;
        if (options) {
            this.asRun = options.asRun;
            this.startDate = options.startDate;
            this.previousEventId = options.previousEventId;
            this.scheduleOrigin = options.scheduleOrigin;
            this.action = options.action;
            this.startMode = options.startMode;
        }
    }
    toString() {
        return `${this.id}\t${this.startDate.toString()}\t${this.comment}`;
    }
    static fromJson(json = {}) {
        return new CommentEvent(json.id, json.comment, {
            title: json.title,
            custom: json.custom,
            asRun: json.asRun ? asRun_1.AsRun.fromJson(json.asRun) : undefined,
            groupId: json.groupId,
            groupName: json.groupName,
            groupType: json.groupType,
            startDate: json.startDate ? bTime_1.BDateTime.fromJson(json.startDate) : undefined,
            previousEventId: json.previousEventId,
            scheduleOrigin: json.scheduleOrigin,
            action: json.action,
            startMode: json.startMode
        });
    }
    toJson() {
        return {
            ...super.toJson(),
            comment: this.comment,
            asRun: this.asRun && this.asRun.toJson(),
            startDate: this.startDate && this.startDate.toJson(),
            previousEventId: this.previousEventId,
            scheduleOrigin: this.scheduleOrigin,
            action: this.action,
            startMode: this.startMode
        };
    }
}
exports.CommentEvent = CommentEvent;
class BreakHeaderEvent extends PrimaryEvent {
    toString() {
        return `${this.id}\t${this.startDate.toString()}\t${this.duration.toString()}\t${this.eventType}\t${this.title}`;
    }
    constructor(id, startMode, options) {
        super(id, "breakHeader", options && {
            title: options.title,
            custom: options.custom,
            customArray: options.customArray,
            groupId: options.groupId,
            groupName: options.groupName,
            groupType: options.groupType,
            comment: options.comment,
            passThrough: options.passThrough,
            previousEventId: options.previousEventId,
            scheduleOrigin: options.scheduleOrigin,
            action: options.action
        });
        this.startMode = startMode;
        this.secondaryEvents = [];
        if (options) {
            this.reconcileId = options.reconcileId;
            this.endMode = options.endMode;
            this.startDate = options.startDate;
            this.originallyScheduledStartTime = options.originallyScheduledStartTime;
            this.duration = options.duration;
            this.timeLinkDeclaration = options.timeLinkDeclaration;
            this.timeLinkReference = options.timeLinkReference;
            this.asRun = options.asRun;
            this.commercials = options.commercials;
            this.blankingEvents = options.blankingEvents;
            this.secondaryEvents = options.secondaryEvents;
            this.hold = options.hold;
            this.previousEventId = options.previousEventId;
            this.scheduleOrigin = options.scheduleOrigin;
            this.action = options.action;
        }
    }
    static getKeys() {
        return [
            { "name": "comment" },
            { "name": "duration", "type": "duration" },
            { "name": "id" },
            { "name": "reconcileId" },
            { "name": "startMode" },
            { "name": "asRun.duration", "type": "duration" },
            { "name": "asRun.startTime", "type": "date" },
            { "name": "asRun.endTime", "type": "date" },
            { "name": "asRun.state" }
        ];
    }
    static fromJson(json = {}) {
        let startDate;
        let duration;
        let secondaryEvents;
        let commercials;
        let asRun;
        let blankingEvents;
        if (json.startDate) {
            startDate = bTime_1.BDateTime.fromJson(json.startDate);
        }
        if (json.duration) {
            duration = bTime_1.BDuration.fromJson(json.duration);
        }
        if (json.secondaryEvents) {
            secondaryEvents = [];
            for (let event of json.secondaryEvents) {
                if (event.eventType == "graphic") {
                    secondaryEvents.push(secondaryEvents_1.GraphicEvent.fromJson(event));
                }
                else if (event.eventType == "voiceover") {
                    secondaryEvents.push(secondaryEvents_1.VoiceoverEvent.fromJson(event));
                }
                else if (event.eventType == "record") {
                    secondaryEvents.push(secondaryEvents_1.RecordEvent.fromJson(event));
                }
                else if (event.eventType == "video") {
                    secondaryEvents.push(secondaryEvents_1.VideoEvent.fromJson(event));
                }
                else if (event.eventType == "gpo") {
                    secondaryEvents.push(gpoEvent_1.GpoEvent.fromJson(event));
                }
                else if (event.eventType == "dve") {
                    secondaryEvents.push(dveEvent_1.DveEvent.fromJson(event));
                }
                else if (event.eventType == "dpi") {
                    secondaryEvents.push(secondaryEvents_1.DpiEvent.fromJson(event));
                }
            }
        }
        if (json.asRun) {
            asRun = asRun_1.AsRun.fromJson(json.asRun);
        }
        if (json.commercials) {
            commercials = [];
            for (let event of json.commercials) {
                commercials.push(PlayEvent.fromJson(event));
            }
        }
        if (json.blankingEvents) {
            blankingEvents = [];
            for (let blankingEvent of json.blankingEvents) {
                blankingEvents.push(blankingEvent_1.BlankingEvent.fromJson(blankingEvent));
            }
        }
        if (json.secondaryEvents) {
            secondaryEvents = [];
            for (let event of json.secondaryEvents) {
                if (event.eventType == "graphic") {
                    secondaryEvents.push(secondaryEvents_1.GraphicEvent.fromJson(event));
                }
                else if (event.eventType == "voiceover") {
                    secondaryEvents.push(secondaryEvents_1.VoiceoverEvent.fromJson(event));
                }
                else if (event.eventType == "record") {
                    secondaryEvents.push(secondaryEvents_1.RecordEvent.fromJson(event));
                }
                else if (event.eventType == "video") {
                    secondaryEvents.push(secondaryEvents_1.VideoEvent.fromJson(event));
                }
                else if (event.eventType == "gpo") {
                    secondaryEvents.push(gpoEvent_1.GpoEvent.fromJson(event));
                }
                else if (event.eventType == "dve") {
                    secondaryEvents.push(dveEvent_1.DveEvent.fromJson(event));
                }
                else if (event.eventType == "dpi") {
                    secondaryEvents.push(secondaryEvents_1.DpiEvent.fromJson(event));
                }
            }
        }
        return new BreakHeaderEvent(json.id, json.startMode, {
            title: json.title,
            custom: json.custom,
            customArray: json.customArray,
            reconcileId: json.reconcileId,
            endMode: json.endMode,
            startDate,
            originallyScheduledStartTime: json.originallyScheduledStartTime,
            duration,
            secondaryEvents,
            commercials,
            asRun,
            timeLinkDeclaration: json.timeLinkDeclaration,
            timeLinkReference: json.timeLinkReference,
            blankingEvents,
            hold: json.hold,
            groupId: json.groupId,
            groupName: json.groupName,
            groupType: json.groupType,
            comment: json.comment,
            passThrough: json.passThrough,
            previousEventId: json.previousEventId,
            scheduleOrigin: json.scheduleOrigin,
            action: json.action
        });
    }
    toJson() {
        return {
            ...super.toJson(),
            reconcileId: this.reconcileId,
            endMode: this.endMode,
            startMode: this.startMode,
            startDate: this.startDate && this.startDate.toJson(),
            originallyScheduledStartTime: this.originallyScheduledStartTime,
            duration: this.duration && this.duration.toJson(),
            timeLinkDeclaration: this.timeLinkDeclaration,
            timeLinkReference: this.timeLinkReference,
            asRun: this.asRun && this.asRun.toJson(),
            commercials: this.commercials ? _.map(this.commercials, commercial => commercial?.toJson()) : undefined,
            blankingEvents: this.blankingEvents && _.map(this.blankingEvents, (blankingEvent) => { return blankingEvent.toJson(); }),
            secondaryEvents: this.secondaryEvents ? _.map(this.secondaryEvents, (secondaryEvent) => { return secondaryEvent.toJson(); }) : [],
            previousEventId: this.previousEventId,
            scheduleOrigin: this.scheduleOrigin,
            action: this.action
        };
    }
}
exports.BreakHeaderEvent = BreakHeaderEvent;
class PlayEvent extends PrimaryEvent {
    toString() {
        return `${this.id}\t${this.startDate.toString()}\t${this.duration.toString()}\t${this.eventType}\t${this.eventCategory}\t${this.title}`;
    }
    constructor(id, startMode, source, options) {
        super(id, "play", options && {
            title: options.title,
            custom: options.custom,
            customArray: options.customArray,
            groupId: options.groupId,
            groupName: options.groupName,
            groupType: options.groupType,
            passThrough: options.passThrough,
            previousEventId: options.previousEventId,
            scheduleOrigin: options.scheduleOrigin,
            action: options.action
        });
        this.startMode = startMode;
        this.source = source;
        if (options) {
            this.enabled = _.has(options, 'enabled') ? options.enabled : true;
            this.title = options.title;
            this.reconcileId = options.reconcileId;
            this.eventId = options.eventId;
            this.eventCategory = options.eventCategory;
            this.groupId = options.groupId;
            this.groupName = options.groupName;
            this.groupType = options.groupType;
            this.comment = options.comment;
            this.aspectRatio = options.aspectRatio;
            this.endMode = options.endMode;
            this.startDate = options.startDate;
            this.originallyScheduledStartTime = options.originallyScheduledStartTime;
            this.duration = options.duration;
            this.programmeId = options.programmeId;
            this.part = options.part;
            this.totalParts = options.totalParts;
            this.lock = options.lock;
            this.secondaryEvents = options.secondaryEvents;
            this.transition = options.transition;
            this.subtitles = options.subtitles;
            this.audioDescriptions = options.audioDescriptions;
            this.blankingEvents = options.blankingEvents;
            this.audioTechnical = options.audioTechnical;
            this.optOut = options.optOut;
            this.asRun = options.asRun;
            this.asRunJobs = options.asRunJobs;
            this.timeLinkDeclaration = options.timeLinkDeclaration;
            this.timeLinkReference = options.timeLinkReference;
            this.hold = options.hold;
            this.passThrough = options.passThrough;
            this.previousEventId = options.previousEventId;
            this.scheduleOrigin = options.scheduleOrigin;
            this.action = options.action;
        }
    }
    static getKeys() {
        return [
            { "name": "aspectRatio" },
            { "name": "audioTechnical.duckEnable" },
            { "name": "audioTechnical.output" },
            { "name": "comment" },
            { "name": "duration", "type": "duration" },
            { "name": "enabled" },
            { "name": "eventCategory" },
            { "name": "eventId" },
            { "name": "eventType" },
            { "name": "id" },
            { "name": "optOut.breakId" },
            { "name": "reconcileId" },
            { "name": "startMode" },
            { "name": "part" },
            { "name": "programmeId" },
            { "name": "totalParts" },
            { "name": "lock" },
            { "name": "source.materialId" },
            { "name": "source.som", type: "date" },
            { "name": "source.source" },
            { "name": "source.sourceType" },
            { "name": "startDate", type: "date" },
            { "name": "startMode" },
            { "name": "title" },
            { "name": "transition.video.duration", "type": "duration" },
            { "name": "transition.video.type" },
            { "name": "asRun.duration", "type": "duration" },
            { "name": "asRun.startTime", "type": "date" },
            { "name": "asRun.endTime", "type": "date" },
            { "name": "asRun.state" }
        ];
    }
    static fromJson(json = {}) {
        let startDate;
        let duration;
        let source;
        let transition;
        let subtitles;
        let audioDescriptions;
        let blankingEvents;
        let secondaryEvents = [];
        let audioTechnical;
        let optOut;
        let asRun;
        let asRunJobs;
        let timeLinkDeclaration;
        let timeLinkReference;
        if (json.startDate) {
            startDate = bTime_1.BDateTime.fromJson(json.startDate);
        }
        if (json.duration) {
            duration = bTime_1.BDuration.fromJson(json.duration);
        }
        if (json.timeLinkDeclaration) {
            timeLinkDeclaration = json.timeLinkDeclaration;
        }
        if (json.timeLinkReference) {
            timeLinkReference = json.timeLinkReference;
        }
        if (json.source) {
            if (json.source.sourceType == "media") {
                source = source_1.MediaSource.fromJson(json.source);
            }
            else if (json.source.sourceType == "router") {
                source = source_1.RouterSource.fromJson(json.source);
            }
        }
        if (json.transition) {
            if (!transition) {
                transition = {};
            }
            if (json.transition.audio) {
                transition.audio = transition_1.AudioTransition.fromJson(json.transition.audio);
            }
            if (json.transition.video) {
                transition.video = transition_1.VideoTransition.fromJson(json.transition.video);
            }
        }
        if (json.subtitles) {
            subtitles = [];
            for (let subtitle of json.subtitles) {
                subtitles.push(subtitle_1.Subtitle.fromJson(subtitle));
            }
        }
        if (json.audioDescriptions) {
            audioDescriptions = [];
            for (let audioDescription of json.audioDescriptions) {
                audioDescriptions.push(audioDescription_1.AudioDescription.fromJson(audioDescription));
            }
        }
        if (json.blankingEvents) {
            blankingEvents = [];
            for (let blankingEvent of json.blankingEvents) {
                blankingEvents.push(blankingEvent_1.BlankingEvent.fromJson(blankingEvent));
            }
        }
        if (json.audioTechnical) {
            audioTechnical = audioTechnical_1.AudioTechnical.fromJson(json.audioTechnical);
        }
        if (json.optOut) {
            optOut = OptOut_1.OptOut.fromJson(json.optOut);
        }
        if (json.asRun) {
            asRun = asRun_1.AsRun.fromJson(json.asRun);
        }
        if (json.asRunJobs) {
            asRunJobs = _.map(json.asRunJobs, (asRunJobs) => asRun_1.AsRunJob.fromJson(asRunJobs));
        }
        if (json.secondaryEvents && (!_.isEmpty(json.secondaryEvents))) {
            secondaryEvents = [];
            for (let event of json.secondaryEvents) {
                if (event.eventType == "graphic") {
                    secondaryEvents.push(secondaryEvents_1.GraphicEvent.fromJson(event));
                }
                else if (event.eventType == "voiceover") {
                    secondaryEvents.push(secondaryEvents_1.VoiceoverEvent.fromJson(event));
                }
                else if (event.eventType == "record") {
                    secondaryEvents.push(secondaryEvents_1.RecordEvent.fromJson(event));
                }
                else if (event.eventType == "video") {
                    secondaryEvents.push(secondaryEvents_1.VideoEvent.fromJson(event));
                }
                else if (event.eventType == "gpo") {
                    secondaryEvents.push(gpoEvent_1.GpoEvent.fromJson(event));
                }
                else if (event.eventType == "dve") {
                    secondaryEvents.push(dveEvent_1.DveEvent.fromJson(event));
                }
                else if (event.eventType == "dpi") {
                    secondaryEvents.push(secondaryEvents_1.DpiEvent.fromJson(event));
                }
            }
        }
        return new PlayEvent(json.id, json.startMode, source, {
            enabled: json.enabled,
            title: json.title,
            custom: json.custom,
            customArray: json.customArray,
            reconcileId: json.reconcileId,
            eventId: json.eventId,
            eventCategory: json.eventCategory,
            groupId: json.groupId,
            groupName: json.groupName,
            groupType: json.groupType,
            comment: json.comment,
            aspectRatio: json.aspectRatio,
            optOut,
            endMode: json.endMode,
            startDate,
            originallyScheduledStartTime: json.originallyScheduledStartTime,
            programmeId: json.programmeId,
            part: json.part,
            totalParts: json.totalParts,
            lock: json.lock,
            passThrough: json.passThrough,
            duration,
            secondaryEvents,
            transition,
            subtitles,
            audioDescriptions,
            blankingEvents,
            audioTechnical,
            asRunJobs,
            asRun,
            timeLinkDeclaration,
            timeLinkReference,
            hold: json.hold,
            previousEventId: json.previousEventId,
            scheduleOrigin: json.scheduleOrigin,
            action: json.action
        });
    }
    toJson() {
        return {
            ...super.toJson(),
            reconcileId: this.reconcileId,
            eventId: this.eventId,
            eventCategory: this.eventCategory,
            groupId: this.groupId,
            groupName: this.groupName,
            groupType: this.groupType,
            comment: this.comment,
            aspectRatio: this.aspectRatio,
            startMode: this.startMode,
            endMode: this.endMode,
            startDate: this.startDate && this.startDate.toJson(),
            originallyScheduledStartTime: this.originallyScheduledStartTime,
            duration: this.duration && this.duration.toJson(),
            programmeId: this.programmeId,
            part: this.part,
            totalParts: this.totalParts,
            lock: this.lock,
            passThrough: this.passThrough,
            source: this.source && this.source.toJson(),
            subtitles: this.subtitles && _.map(this.subtitles, (subtitle) => { return subtitle.toJson(); }),
            audioDescriptions: this.audioDescriptions && _.map(this.audioDescriptions, (audioDescription) => { return audioDescription.toJson(); }),
            blankingEvents: this.blankingEvents && _.map(this.blankingEvents, (blankingEvent) => { return blankingEvent.toJson(); }),
            audioTechnical: this.audioTechnical && this.audioTechnical.toJson(),
            optOut: this.optOut && this.optOut.toJson(),
            secondaryEvents: this.secondaryEvents ? _.map(this.secondaryEvents, (secondaryEvent) => { return secondaryEvent.toJson(); }) : [],
            transition: {
                audio: _.get(this.transition, "audio") && this.transition.audio.toJson(),
                video: _.get(this.transition, "video") && this.transition.video.toJson(),
            },
            asRun: this.asRun && this.asRun.toJson(),
            asRunJobs: this.asRunJobs && _.map(this.asRunJobs, (asRunJobs) => { return asRunJobs.toJson(); }),
            timeLinkReference: this.timeLinkReference,
            timeLinkDeclaration: this.timeLinkDeclaration,
            previousEventId: this.previousEventId,
            scheduleOrigin: this.scheduleOrigin,
            action: this.action
        };
    }
}
exports.PlayEvent = PlayEvent;
