"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DveEvent = void 0;
const secondaryEvent_1 = require("./secondaryEvent");
const bTime_1 = require("../../bTime");
const source_1 = require("../source");
class DveEvent extends secondaryEvent_1.SecondaryEventBase {
    constructor(id, startMode, _secondarySource, _secondaryPosition, _rate, options) {
        super("dve", startMode, id, options);
        this._secondarySource = _secondarySource;
        this._secondaryPosition = _secondaryPosition;
        this._rate = _rate;
        this._primaryPosition = options.primaryPosition;
    }
    get primaryPosition() {
        return this._primaryPosition;
    }
    get secondarySource() {
        return this._secondarySource;
    }
    get secondaryPosition() {
        return this._secondaryPosition;
    }
    get rate() {
        return this._rate;
    }
    static getKeys() {
        return [
            { "name": "primaryPosition" },
            { "name": "secondaryPosition" },
            { "name": "rate" },
            { "name": "secondarySource" }
        ];
    }
    static fromJson(json = {}) {
        let startOffset, endOffset, duration, source;
        if (json.startOffset) {
            startOffset = bTime_1.BDuration.fromJson(json.startOffset);
        }
        if (json.endOffset) {
            endOffset = bTime_1.BDuration.fromJson(json.endOffset);
        }
        if (json.duration) {
            duration = bTime_1.BDuration.fromJson(json.duration);
        }
        if (json.secondarySource?.sourceType == "media") {
            source = source_1.MediaSource.fromJson(json.secondarySource);
        }
        else if (json.secondarySource?.sourceType == "router") {
            source = source_1.RouterSource.fromJson(json.secondarySource);
        }
        return new DveEvent(json.id, json.startMode, source, json.secondaryPosition, json.rate, {
            primaryPosition: json.primaryPosition,
            startOffset,
            startOffsetType: json.startOffsetType,
            endMode: json.endMode,
            endOffset,
            duration,
            custom: json.custom,
            comment: json.comment,
            passThrough: json.passThrough,
            asRun: json.asRun
        });
    }
    toJson() {
        return {
            ...super.toJson(),
            primaryPosition: this.primaryPosition,
            secondaryPosition: this.secondaryPosition,
            secondarySource: this.secondarySource.toJson(),
            rate: this.rate
        };
    }
}
exports.DveEvent = DveEvent;
