"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphicEvent = exports.LiveInput = exports.ClipInput = exports.GraphicInput = exports.GraphicTransition = exports.FieldInfo = void 0;
const secondaryEvent_1 = require("./secondaryEvent");
const bTime_1 = require("../../bTime");
const _ = require("lodash");
class FieldInfo {
    static getKeys() {
        return [
            { "name": "name" },
            { "name": "value" }
        ];
    }
}
exports.FieldInfo = FieldInfo;
class GraphicTransition {
    constructor(options) {
        if (options) {
            this.inDuration = options.inDuration;
            this.outDuration = options.outDuration;
        }
    }
    static fromJson(json = {}) {
        let inDuration, outDuration;
        if (json.inDuration) {
            inDuration = bTime_1.BDuration.fromJson(json.inDuration);
        }
        if (json.outDuration) {
            outDuration = bTime_1.BDuration.fromJson(json.outDuration);
        }
        return new GraphicTransition({ inDuration, outDuration });
    }
    toJson() {
        return {
            inDuration: this.inDuration && this.inDuration.toJson(),
            outDuration: this.outDuration && this.outDuration.toJson()
        };
    }
}
exports.GraphicTransition = GraphicTransition;
class GraphicInput {
    toJson() {
        return {
            ...this,
            startTimeOffset: this.startTimeOffset?.toJson(),
            endTimeOffset: this.endTimeOffset?.toJson(),
            duration: this.duration?.toJson()
        };
    }
}
exports.GraphicInput = GraphicInput;
function getDuration(time) {
    if (time?.duration) {
        return bTime_1.BDuration.fromJson(time);
    }
    else if (time?.time) {
        const dateTime = time;
        return bTime_1.BDuration.fromTcString(dateTime.time, dateTime.frameRate);
    }
}
class ClipInput extends GraphicInput {
    toJson() {
        const graphicInput = super.toJson();
        const clipInput = graphicInput;
        clipInput.som = this.som?.toJson();
        return clipInput;
    }
    static fromJson(json) {
        const clipInput = new ClipInput();
        clipInput.id = json.id;
        clipInput.startMode = json.startMode;
        clipInput.startTimeOffset = getDuration(json.startTimeOffset);
        clipInput.endTimeOffset = getDuration(json.endTimeOffset);
        clipInput.duration = getDuration(json.duration);
        clipInput.hasAudio = json.hasAudio;
        clipInput.source = json.source;
        clipInput.sourceType = json.sourceType;
        clipInput.input = json.input;
        clipInput.materialId = json.materialId;
        clipInput.som = bTime_1.BDateTime.fromJson(json.som);
        clipInput.somType = json.somType;
        return clipInput;
    }
}
exports.ClipInput = ClipInput;
class LiveInput extends GraphicInput {
    toJson() {
        const graphicInput = super.toJson();
        const liveInput = graphicInput;
        return liveInput;
    }
    static fromJson(json) {
        const liveInput = new LiveInput();
        liveInput.id = json.id;
        liveInput.startMode = json.startMode;
        liveInput.startTimeOffset = getDuration(json.startTimeOffset);
        liveInput.endTimeOffset = getDuration(json.endTimeOffset);
        liveInput.duration = getDuration(json.duration);
        liveInput.hasAudio = json.hasAudio;
        liveInput.source = json.source;
        liveInput.sourceType = json.sourceType;
        liveInput.input = json.input;
        return liveInput;
    }
}
exports.LiveInput = LiveInput;
class GraphicEvent extends secondaryEvent_1.SecondaryEventBase {
    constructor(id, startMode, page, fields, options) {
        super("graphic", startMode, id, options);
        this.page = page;
        this.fields = fields;
        if (options) {
            this.template = options.template;
            this.overrideEventType = options.overrideEventType;
            this._animated = options.animated;
            this.loop = options.loop;
            this._animationOffset = options.animationOffset;
            this._layer = options.layer;
            this._graphicsAttributes = options.graphicsAttributes;
            this._clipInputs = options.clipInputs;
            this._liveInputs = options.liveInputs;
            this._transition = options.transition;
            this._hasAudio = options.hasAudio;
        }
    }
    get animated() {
        return this._animated;
    }
    get animationOffset() {
        return this._animationOffset;
    }
    get layer() {
        return this._layer;
    }
    set layer(layer) {
        this._layer = layer;
    }
    get clipInputs() {
        return this._clipInputs;
    }
    static getKeys() {
        return [
            { "name": "page" },
            { "name": "template" },
            { "name": "overrideEventType" },
            { "name": "fields", "type": "array" }
        ];
    }
    static getFieldKeys() {
        return [
            { "name": "name" },
            { "name": "value" }
        ];
    }
    static fromJson(json = {}) {
        let startOffset, endOffset, duration, animationOffset;
        if (json.startOffset) {
            startOffset = bTime_1.BDuration.fromJson(json.startOffset);
        }
        if (json.endOffset) {
            endOffset = bTime_1.BDuration.fromJson(json.endOffset);
        }
        if (json.duration) {
            duration = bTime_1.BDuration.fromJson(json.duration);
        }
        if (json.animationOffset) {
            animationOffset = bTime_1.BDuration.fromJson(json.animationOffset);
        }
        const orderedFields = (json.fields && _.isArray(json.fields)) ? json.fields.sort((a, b) => {
            return a.name.localeCompare(b.name, undefined, { numeric: true });
        }) : undefined;
        return new GraphicEvent(json.id, json.startMode, json.page, _.filter(orderedFields, (field) => { return field.value; }), {
            action: json.action,
            startMode: json.startMode,
            title: json.title,
            startOffset,
            startOffsetType: json.startOffsetType,
            endMode: json.endMode,
            endOffset,
            duration,
            template: json.template,
            custom: json.custom,
            comment: json.comment,
            overrideEventType: json.overrideEventType,
            transition: GraphicTransition.fromJson(json.transition),
            animated: json.animated,
            animationOffset,
            loop: json.loop,
            layer: json.layer,
            graphicsAttributes: json.graphicsAttributes,
            passThrough: json.passThrough,
            clipInputs: json.clipInputs ? _.map(json.clipInputs, (clipInput) => { return ClipInput.fromJson(clipInput); }) : undefined,
            liveInputs: json.liveInputs ? _.map(json.liveInputs, (liveInput) => { return LiveInput.fromJson(liveInput); }) : undefined,
            hasAudio: json.hasAudio,
            asRun: json.asRun
        });
    }
    toJson() {
        const orderedFields = this.fields && _.isArray(this.fields) ? this.fields.sort((a, b) => {
            return a.name.localeCompare(b.name, undefined, { numeric: true });
        }) : undefined;
        return {
            ...super.toJson(),
            fields: _.filter(orderedFields, (field) => { return field.value; }),
            page: this.page,
            template: this.template,
            overrideEventType: this.overrideEventType,
            animated: this._animated,
            loop: this.loop,
            layer: this.layer,
            graphicsAttributes: this._graphicsAttributes,
            clipInputs: _.map(this._clipInputs, (clipInput) => {
                return clipInput.toJson();
            }),
            liveInputs: _.map(this._liveInputs, (liveInput) => {
                return liveInput.toJson();
            }),
            passThrough: this.passThrough,
            transition: {
                inDuration: this._transition?.inDuration?.toJson(),
                outDuration: this._transition?.outDuration?.toJson()
            },
            animationOffset: this.animationOffset?.toJson(),
            hasAudio: this._hasAudio,
            action: this.action,
        };
    }
}
exports.GraphicEvent = GraphicEvent;
