"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouterSource = exports.MediaSource = exports.Source = void 0;
const bTime_1 = require("../bTime");
class Source {
    constructor(source, sourceType) {
        this.source = source;
        this.sourceType = sourceType;
    }
    toJson() {
        return {
            source: this.source,
            sourceType: this.sourceType
        };
    }
}
exports.Source = Source;
class MediaSource extends Source {
    constructor(source, materialId, som, options) {
        super(source, 'media');
        this.materialId = materialId;
        this.som = som;
        if (options) {
            this.duration = options.duration;
            this.somType = options.somType;
        }
    }
    static fromJson(json = {}) {
        let som;
        if (json.som) {
            som = bTime_1.BDateTime.fromJson(json.som);
        }
        return new MediaSource(json.source, json.materialId, som, {
            duration: json.duration && bTime_1.BDuration.fromJson(json.duration),
            somType: json.somType
        });
    }
    toJson() {
        return {
            ...super.toJson(),
            materialId: this.materialId,
            som: this.som && this.som.toJson(),
            somType: this.somType,
            duration: this.duration && this.duration.toJson()
        };
    }
}
exports.MediaSource = MediaSource;
class RouterSource extends Source {
    constructor(source, options) {
        super(source, 'router');
        if (options) {
            this.materialId = options.materialId;
        }
    }
    static fromJson(json = {}) {
        return new RouterSource(json.source, { materialId: json.materialId });
    }
    toJson() {
        return {
            ...super.toJson(),
            materialId: this.materialId
        };
    }
}
exports.RouterSource = RouterSource;
