"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InterfaceUser = void 0;
const _ = require("lodash");
class InterfaceUser {
    static get() {
        return InterfaceUser.__instance;
    }
    constructor(user, storeMetadata) {
        this.user = user;
        this.storeMetadata = storeMetadata;
        this.onUserMetadataCallbacks = new Set();
        InterfaceUser.__instance = this;
    }
    getUser() {
        return this.user;
    }
    getMetadata(key) {
        if (!key) {
            return this.user.details?.userMetadata;
        }
        return _.get(this.user.details?.userMetadata, key);
    }
    setMetadata(key, value, store = true) {
        const oldValue = this.getMetadata(key);
        if (store) {
            try {
                this.storeMetadata?.(key, value, oldValue);
            }
            catch { }
        }
        _.set(this.user.details.userMetadata, key, value);
        this.onUserMetadataCallbacks.forEach((fn) => {
            try {
                fn?.(key, value, oldValue);
            }
            catch { }
        });
    }
    onUserMetadata(eventHandler) {
        this.onUserMetadataCallbacks.add(eventHandler);
        return {
            dispose: () => {
                this.offUserMetadata(eventHandler);
            }
        };
    }
    offUserMetadata(eventHandler) {
        this.onUserMetadataCallbacks.delete(eventHandler);
    }
}
exports.InterfaceUser = InterfaceUser;
