"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectorOperations = void 0;
class SelectorOperations {
    static get() {
        if (!this.__instance) {
            this.__instance = new SelectorOperations();
        }
        return this.__instance;
    }
    constructor() {
        this.options = { timezone: "UTC", shoppingBasket: { enabled: false }, debugMode: false };
        this.setup = (operations, options) => {
            this.operations = operations;
            this.options = options ?? { timezone: "UTC", shoppingBasket: { enabled: false }, debugMode: false };
            if (!this.options.timezone) {
                this.options.timezone = "UTC";
            }
        };
        this.checkSetup = () => {
            if (!this.operations) {
                throw new Error("Operations have not been configured, please check your setup.");
            }
        };
        this.getFileLink = (type, fileName, location, path = "/") => {
            return this.operations.getFileLink(type, fileName, location, path);
        };
        this.getOptions = () => {
            return this.options;
        };
        this.getShoppingBasket = () => {
            return this.options.shoppingBasket;
        };
        this.selectApp = (id, data) => {
            this.checkSetup();
            return this.operations.selectApp(id, data);
        };
        this.askModalQuestion = (question, resultButtons, inputOptions) => {
            this.checkSetup();
            return this.operations.askModalQuestion(question, resultButtons, inputOptions);
        };
        this.getModalForm = (form) => {
            this.checkSetup();
            return this.operations.getModalForm(form);
        };
        this.getModalConfirmation = (question, yesText, noText) => {
            this.checkSetup();
            return this.operations.getModalConfirmation(question, yesText, noText);
        };
        this.sendNotification = (title, body, params) => {
            this.checkSetup();
            return this.operations.sendNotification(title, body, params);
        };
        this.startAudioAlert = (alert) => {
            this.checkSetup();
            return this.operations.startAudioAlert(alert);
        };
        this.stopAudioAlert = (alert) => {
            this.checkSetup();
            return this.operations.stopAudioAlert(alert);
        };
        this.getSearchDetail = (searchType, result) => {
            this.checkSetup();
            return this.operations.getSearchDetail(searchType, result);
        };
        this.showSearchResults = (state) => {
            this.checkSetup();
            return this.operations.showSearchResults(state);
        };
        this.raiseToast = (type, title, description, options) => {
            this.checkSetup();
            return this.operations.raiseToast(type, title, description, options);
        };
        this.setUserMetadata = (key, value) => {
            this.checkSetup();
            return this.operations.setUserMetadata(key, value);
        };
        this.getUserMetadata = (key) => {
            this.checkSetup();
            return this.operations.getUserMetadata(key);
        };
        this.getSessionId = () => {
            this.checkSetup();
            return this.operations.getSessionId();
        };
        this.getAvailableApplications = () => {
            this.checkSetup();
            return this.operations.getAvailableApplications();
        };
        window.__SELECTOR_OPERATIONS__ = this;
    }
    showJobModal(jobUuid, options) {
        this.checkSetup();
        return this.operations.showJobModal(jobUuid, options);
    }
    hideJobModal() {
        this.checkSetup();
        return this.operations.hideJobModal();
    }
}
exports.SelectorOperations = SelectorOperations;
