"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonJobBuilder = void 0;
const JsonInfoBlock_1 = require("../infoBlocks/json/JsonInfoBlock");
const RestoreInfoBlock_1 = require("../infoBlocks/restore/RestoreInfoBlock");
const EdlInfoBlock_1 = require("../infoBlocks/edl/EdlInfoBlock");
const AttachmentInfoBlock_1 = require("../infoBlocks/attachment/AttachmentInfoBlock");
const NoteInfoBlock_1 = require("../infoBlocks/note/NoteInfoBlock");
const FileLocatorListInfoBlock_1 = require("../infoBlocks/fileLocatorList/FileLocatorListInfoBlock");
const FormInfoBlock_1 = require("../infoBlocks/form/FormInfoBlock");
const GenericInfoBlock_1 = require("../infoBlocks/generic/GenericInfoBlock");
const emailInfoBlock_1 = require("../infoBlocks/email/emailInfoBlock");
const ResourcesInfoBlock_1 = require("../infoBlocks/resources/ResourcesInfoBlock");
const MamItemInfoBlock_1 = require("../infoBlocks/mamItem/MamItemInfoBlock");
const SurveyInfoBlock_1 = require("../infoBlocks/survey/SurveyInfoBlock");
class CommonJobBuilder {
    constructor(job) {
        this.job = job || {};
    }
    addInfoBlock(infoType, subType) {
        return new GenericInfoBlock_1.GenericInfoBlock({ infoType: infoType, subType: subType }, this);
    }
    addJsonInfoBlock(subType) {
        return new JsonInfoBlock_1.JsonInfoBlock({ subType: subType }, this);
    }
    addEmailInfoBlock(subType) {
        return new emailInfoBlock_1.EmailInfoBlock({ subType: subType }, this);
    }
    addFormInfoBlock(subType) {
        return new FormInfoBlock_1.FormInfoBlock({ subType: subType }, this);
    }
    addRestoreInfoBlock(subType) {
        return new RestoreInfoBlock_1.RestoreInfoBlock({ subType: subType }, this);
    }
    addEdlInfoBlock(subType) {
        return new EdlInfoBlock_1.EdlInfoBlock({ subType: subType }, this);
    }
    addAttachmentInfoBlock(subType) {
        return new AttachmentInfoBlock_1.AttachmentInfoBlock({ subType: subType }, this);
    }
    addNoteInfoBlock(subType = '') {
        return new NoteInfoBlock_1.NoteInfoBlock({ subType: subType }, this);
    }
    addFileLocatorListInfoBlock(subType = '') {
        return new FileLocatorListInfoBlock_1.FileLocatorListInfoBlock({ subType: subType }, this);
    }
    addResourcesInfoBlock(subType) {
        return new ResourcesInfoBlock_1.ResourcesInfoBlock({ subType: subType }, this);
    }
    addMamItemsInfoBlock(subType) {
        return new MamItemInfoBlock_1.MamItemsInfoBlock({ subType: subType }, this);
    }
    addSurveyInfoBlock(subType) {
        return new SurveyInfoBlock_1.SurveyInfoBlock({ subType: subType }, this);
    }
    internalRef(internalRef) {
        this.job.internalRef = internalRef;
        return this;
    }
    externalRef(externalRef) {
        this.job.externalRef = externalRef;
        return this;
    }
    description(description) {
        this.job.description = description;
        return this;
    }
    workflowName(workflowName) {
        this.job.workflowName = workflowName;
        return this;
    }
    workflow(workflow) {
        this.job.workflow = workflow;
        return this;
    }
    priority(value) {
        this.job.priority = value;
        return this;
    }
    completeBy(value) {
        this.job.completeBy = value;
        return this;
    }
    uuid(uuid) {
        this.job.uuid = uuid;
        return this;
    }
    streamKey(streamKey) {
        this.job.streamKey = streamKey;
        return this;
    }
    altRef(key, value) {
        this.job.altRefs ??= {};
        this.job.altRefs[key] ??= [];
        if (typeof value === "string") {
            value = [value];
        }
        this.job.altRefs[key].push(...value);
        return this;
    }
    setUser(user) {
        this.user = user;
        return this;
    }
    getUser() {
        return this.user;
    }
    get() {
        return this.job;
    }
    addChild(child) {
        if (!this.job.children) {
            this.job.children = [];
        }
        this.job.children.push(child);
        return this;
    }
}
exports.CommonJobBuilder = CommonJobBuilder;
