"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsperaShareSubscriber = void 0;
const interfaces_1 = require("./interfaces");
class AsperaShareSubscriber extends interfaces_1.AbstractSubscriber {
    /**
     * Gets all users from Aspera Shares
     * @returns {AsperaSharesUser[]}
     */
    getAllUsers() {
        return this.send('getAllUsers');
    }
    /**
     * Gets a user by ID from Aspera Shares
     * @param id - ID of the Aspera Share user
     * @returns {AsperaSharesUser}
     */
    getUserById(id) {
        return this.send('getUserById', { id });
    }
    /**
     * Adds a new local user to Aspera Shares
     * @param userDetails {AsperaSharesUser}
     * @returns {AsperaSharesUser}
     */
    addLocalUser(userDetails) {
        return this.send('addUser', {
            userDetails
        });
    }
    /**
     * Gets all nodes from Aspera Shares
     * @returns {AsperaSharesNode[]}
     */
    getAllNodes() {
        return this.send('getAllNodes');
    }
    /**
     * Gets a node by ID from Aspera Shares
     * @param id - ID of the Aspera Share node
     * @returns {AsperaSharesNode}
     */
    getNodeById(id) {
        return this.send('getNodeById', { id });
    }
    /**
     * Gets all shares from Aspera Shares
     * @returns {AsperaSharesShare[]}
     */
    getAllShares() {
        return this.send('getAllShares');
    }
    /**
     * Gets a share by ID from Aspera Shares
     * @param id - ID of the Aspera Share node
     * @returns {AsperaSharesShare}
     */
    getShareById(id) {
        return this.send('getShareById', { id });
    }
    /**
     * Gets user permissions for an Aspera Shares share
     * @returns {AsperaSharesUserPermission[]}
     */
    getUserPermissionsForShare(shareId) {
        return this.send('getUserPermissionsForShare', { shareId });
    }
    /**
     * Authorises local user to an Aspera Shares share
     * @param sharesUserPermission {AsperaSharesUserPermission}
     * @returns {AsperaSharesUserPermission}
     */
    authorizeUserToShare(sharesUserPermission) {
        return this.send('authoriseUserToShare', sharesUserPermission);
    }
}
exports.AsperaShareSubscriber = AsperaShareSubscriber;
