"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthenticationServiceSubscriber = void 0;
const _ = require("lodash");
const interfaces_1 = require("./interfaces");
class AuthenticationServiceSubscriber extends interfaces_1.AbstractSubscriber {
    constructor() {
        super(...arguments);
        this.onAddUser = (msg) => {
            this.cachedAllUserResult.getResult().users.push(msg.getParams().user);
        };
        this.onUpdateUser = (msg) => {
            this.cachedAllUserResult.getResult().users.splice(this.cachedAllUserResult.getResult().users.findIndex(u => u.username === msg.getParams().user.username), 1, msg.getParams().user);
        };
        this.onDeleteUser = (msg) => {
            this.cachedAllUserResult.getResult().users.splice(this.cachedAllUserResult.getResult().users.findIndex(u => u.username === msg.getParams().username), 1);
        };
    }
    addUser(params) {
        params.password = Buffer.from(params.password).toString('base64');
        return this.send('addUser', params);
    }
    addGroup(params) {
        return this.send('addGroup', params);
    }
    deleteGroup(params) {
        return this.send('deleteGroup', params);
    }
    updateUser(params) {
        if (params.password) {
            params.password = Buffer.from(params.password).toString('base64');
        }
        return this.send('updateUser', params);
    }
    updateUserMetadata(params) {
        return this.send('updateUserMetadata', params);
    }
    mergeUserMetadata(params) {
        return this.send('mergeUserMetadata', params);
    }
    deleteUser(params) {
        return this.send('deleteUser', params);
    }
    getAllPermissions() {
        return this.send("getAllPermissions");
    }
    async getAllUsers() {
        if (this.cachedAllUserResult) {
            return this.cachedAllUserResult;
        }
        const result = await this.send("getAllUsers");
        if (!result.isError()) {
            this.cachedAllUserResult = result;
            this.status.addUser(this.onAddUser);
            this.status.updateUser(this.onUpdateUser);
            this.status.deleteUser(this.onDeleteUser);
        }
        return result;
    }
    getAllGroups() {
        return this.send("getAllGroups");
    }
    async getUsersInGroup(group) {
        const allUsers = (await this.getAllUsers()).getResult().users;
        return _.filter(allUsers, (user) => {
            return _.includes(user.groups, group);
        });
    }
    authenticateUser(params) {
        if (!params.oauth) {
            params.password = Buffer.from(params.password).toString('base64');
        }
        return this.send('authenticateUser', params);
    }
    refreshSession(params) {
        return this.send('refreshSession', params);
    }
    removeSession(params) {
        return this.send('removeSession', params);
    }
    getUserPermissions(params) {
        return this.send('getUserPermissions', params);
    }
    getUserPermissionsFromSessionId(params) {
        return this.send('getUserPermissionsFromSessionId', params);
    }
    getUser(params) {
        return this.send('getUser', params);
    }
    getUserMetadata(params) {
        return this.send('getUserMetadata', params);
    }
    getActiveLogins() {
        return this.send('getActiveLogins');
    }
    addGroupPermissionLink(params) {
        return this.send('addGroupPermissionLink', params);
    }
    deleteGroupPermissionLink(params) {
        return this.send('deleteGroupPermissionLink', params);
    }
    addUserGroupLink(params) {
        return this.send('addUserGroupLink', params);
    }
    deleteUserGroupLink(params) {
        return this.send('deleteUserGroupLink', params);
    }
    changePassword(params) {
        params.currentPassword = Buffer.from(params.currentPassword).toString('base64');
        params.newPassword = Buffer.from(params.newPassword).toString('base64');
        return this.send('changePassword', params);
    }
}
exports.AuthenticationServiceSubscriber = AuthenticationServiceSubscriber;
