"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BigDataSubscriber = void 0;
const _ = require("lodash");
const interfaces_1 = require("./interfaces");
class BigDataSubscriber extends interfaces_1.AbstractSubscriber {
    constructor(serviceName, providerId, loginDetails, serviceId, statusProcessor, subscriber = 'subscriber', //you can provide your own subscriber here if you wanted to wrap the real subscriber in some way (e.g. websockets)
    subscriberOptions) {
        super(serviceName, loginDetails, statusProcessor, subscriber, //you can provide your own subscriber here if you wanted to wrap the real subscriber in some way (e.g. websockets)
        subscriberOptions);
        this.providerId = providerId;
        this.serviceId = serviceId;
    }
    async insert(table, id, data, exactTime) {
        const params = { objectPath: id, table: table, data: data };
        if (exactTime) {
            params.exactTime = exactTime;
        }
        return await this.insertData(params);
    }
    async insertData(params) {
        try {
            if (await this.connect()) {
                const result = await this.sendControl('insertData', {
                    ...JSON.parse(JSON.stringify(params, (key, value) => {
                        if (_.isString(value)) {
                            return value.replace(/'/g, "''");
                        }
                        else {
                            return value;
                        }
                    })),
                    service: this.serviceId,
                    instance: this.providerId
                }, { timeoutSecs: 3 });
                return !result.isError();
            }
            else {
                return false;
            }
        }
        catch (e) {
            return false;
        }
    }
    async insertDataArray(params) {
        try {
            if (await this.connect()) {
                const result = await this.sendControl('insertDataArray', {
                    ...JSON.parse(JSON.stringify(params, (key, value) => {
                        if (_.isString(value)) {
                            return value.replace(/'/g, "''");
                        }
                        else {
                            return value;
                        }
                    })),
                    service: this.serviceId,
                    instance: this.providerId
                }, { timeoutSecs: 3 });
                return !result.isError();
            }
            else {
                return false;
            }
        }
        catch (e) {
            return false;
        }
    }
    getTables() {
        return this.send('getTables');
    }
    getReportList() {
        return this.send('getReportList');
    }
    getFormForReport(params) {
        return this.send('getFormForReport', params);
    }
    startReport(params) {
        return this.send('startReport', params);
    }
    getReport(params) {
        return this.send('getReport', params);
    }
    query(params) {
        if (params.filters && params.filter) {
            params.filters.push(params.filter);
            delete params.filter;
        }
        return this.send('query', params);
    }
}
exports.BigDataSubscriber = BigDataSubscriber;
