"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BusbyCertificateManagerSubscriber = void 0;
const AbstractSubscriber_1 = require("./interfaces/AbstractSubscriber");
const SubscriberManager_1 = require("./interfaces/SubscriberManager");
class BusbyCertificateManagerSubscriber extends AbstractSubscriber_1.AbstractSubscriber {
    constructor() {
        super(...arguments);
        this.instances = {};
    }
    static get(_1, _2, _3, _4) {
        return SubscriberManager_1.SubscriberManager.get().getSubscriber("busbyCertificateManager", this);
    }
    static async getConnected() {
        return await SubscriberManager_1.SubscriberManager.get().getConnectedSubscriber("busbyCertificateManager", this);
    }
    async getRootCertificate() {
        if (!this.rootCA) {
            await this.waitUntilConnected();
            this.rootCA = (await this.control.getCACertificate())?.getResult();
        }
        return this.rootCA;
    }
    async getCertificates(providerId, altNames, serviceId) {
        const os = await Promise.resolve().then(() => require("os"));
        const results = [
            os.hostname(),
            os.hostname().split(".")[0],
            "127.0.0.1",
            "localhost"
        ];
        const nets = os.networkInterfaces();
        for (const name of Object.keys(nets)) {
            for (const net of nets[name]) {
                if (net.family === 'IPv4' && !net.internal) {
                    results.push(net.address);
                }
            }
        }
        const alternativeNames = altNames.concat(results);
        await this.waitUntilConnected();
        this.instances[providerId] = (await this.control.getCertificate({
            providerId,
            alternativeNames,
            serviceId,
        }))?.getResult();
        return this.instances[providerId];
    }
}
exports.BusbyCertificateManagerSubscriber = BusbyCertificateManagerSubscriber;
