"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceSubscriber = exports.ResourceGroupSubscriber = void 0;
const interfaces_1 = require("./interfaces");
class ResourceGroupSubscriber extends interfaces_1.AbstractSubscriber {
    get resource() {
        let _changer = "";
        return new Proxy({}, {
            get: ({}, resourceType) => {
                if (resourceType === "_setChanger") {
                    return (changer) => {
                        _changer = changer;
                    };
                }
                return {
                    get: (params) => {
                        if ("resourceIds" in params) {
                            return this.control.getResources({ ...params, resourceType });
                        }
                        return this.control.getResource({ ...params, resourceType });
                    },
                    query: (params) => {
                        return this.control.queryResources({ ...params, resourceType });
                    },
                    queryAll: async (params) => {
                        let results = [];
                        let res = await this.control.queryResources({ ...params, resourceType, offset: null });
                        if (!res.isError()) {
                            results.push(...res.getResult().resources);
                            while (!res.isError() && res.getResult().hasMoreRows) {
                                res = await this.control.queryResources({ ...params, resourceType, offset: results.length - 1 });
                                results.push(...res.getResult().resources);
                            }
                        }
                        res.getResult().resources = results;
                        return res;
                    },
                    add: (params) => {
                        if ("resources" in params) {
                            return this.control.addResources({ changer: _changer, ...params, resourceType, });
                        }
                        return this.control.addResource({ changer: _changer, ...params, resourceType, });
                    },
                    addAll: (params) => {
                        return this.control.addResources({ changer: _changer, ...params, resourceType, });
                    },
                    update: (params) => {
                        return this.control.updateResource({ changer: _changer, ...params, resourceType, });
                    },
                    updateAll: (params) => {
                        return this.control.updateResources({ changer: _changer, ...params, resourceType, });
                    },
                    merge: (params) => {
                        return this.control.mergeResource({ changer: _changer, ...params, resourceType, });
                    },
                    mergeAll: (params) => {
                        return this.control.mergeResources({ changer: _changer, ...params, resourceType, });
                    },
                    delete: (params) => {
                        return this.control.deleteResource({ changer: _changer, ...params, resourceType, });
                    },
                    addUpdate: (params) => {
                        return this.control.addUpdateResource({ changer: _changer, ...params, resourceType, });
                    },
                    addUpdateAll: (params) => {
                        return this.control.addUpdateResources({ changer: _changer, ...params, resourceType });
                    },
                    addMerge: (params) => {
                        return this.control.addMergeResource({ changer: _changer, ...params, resourceType, });
                    },
                    addMergeAll: (params) => {
                        return this.control.addMergeResources({ changer: _changer, ...params, resourceType, });
                    },
                    properties: () => {
                        return this.control.getResourceProperties({ resourceType, });
                    },
                    schema: () => {
                        return this.control.getJsonFormSchema({ resourceType, });
                    }
                };
            }
        });
    }
    getGraphQLSchema(params) {
        return this.send('getGraphQLSchema', params);
    }
    getResourceTypes() {
        return this.send('getResourceTypes');
    }
    getResource(params) {
        return this.send('getResource', params);
    }
    getResources(params) {
        return this.send('getResources', params);
    }
    deleteResource(params) {
        return this.send('deleteResource', params);
    }
    addResource(params) {
        return this.send('addResource', params);
    }
    addResources(params) {
        return this.send('addResources', params);
    }
    updateResource(params) {
        return this.send('updateResource', params);
    }
    updateResources(params) {
        return this.send('updateResources', params);
    }
    addUpdateResource(params) {
        return this.send('addUpdateResource', params);
    }
    addUpdateResources(params) {
        return this.send('addUpdateResources', params);
    }
    mergeResource(params) {
        return this.send('mergeResource', params);
    }
    mergeResources(params) {
        return this.send('mergeResources', params);
    }
    addMergeResource(params) {
        return this.send('addMergeResource', params);
    }
    addMergeResources(params) {
        return this.send('addMergeResources', params);
    }
    getJsonFormSchema(params) {
        return this.send('getJsonFormSchema', params);
    }
    runGraphQLQuery(params) {
        return this.send('runGraphQLQuery', params);
    }
    queryResources(params) {
        return this.send('queryResources', params);
    }
    getResourceProperties(params) {
        return this.send('getResourceProperties', params);
    }
    getResourceDisplayName(params) {
        return this.send('getResourceDisplayName', params);
    }
    getResourceLock(params) {
        return this.send('getResourceLock', params);
    }
    setResourceLock(params) {
        return this.send('setResourceLock', params);
    }
    releaseResourceLock(params) {
        return this.send('releaseResourceLock', params);
    }
    addSnapshot(params) {
        return this.send('addSnapshot', params);
    }
    updateSnapshot(params) {
        return this.send('updateSnapshot', params);
    }
    deleteSnapshot(params) {
        return this.send('deleteSnapshot', params);
    }
    getAllSnapshots() {
        return this.send('getAllSnapshots');
    }
}
exports.ResourceGroupSubscriber = ResourceGroupSubscriber;
class ResourceSubscriber {
    static get(serviceName, statusProcessor, subscriberOptions, ...args) {
        return ResourceGroupSubscriber.get(serviceName, statusProcessor, subscriberOptions, ...args);
    }
    static async getConnected(serviceName, statusProcessor, subscriberOptions, ...args) {
        return ResourceGroupSubscriber.getConnected(serviceName, statusProcessor, subscriberOptions, ...args);
    }
}
exports.ResourceSubscriber = ResourceSubscriber;
