"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JobBuilder = exports.StateStoreSubscriber = void 0;
const _ = require("lodash");
const uuid = require("uuid");
const interfaces_1 = require("./interfaces");
const esbInterface_1 = require("../esbInterface");
const builder_1 = require("../stateStore/builder");
const async_mutex_1 = require("async-mutex");
// Reexport so as not to break things importing from this subscriber
__exportStar(require("../stateStore/model"), exports);
class StateStoreSubscriber extends interfaces_1.AbstractSubscriber {
    constructor() {
        super(...arguments);
        this.jobWrapperQueue = [];
        this.onVisibilityChanged = () => {
            if (document.hidden) {
                this.jobWrapperTimeout && clearTimeout(this.jobWrapperTimeout);
            }
            else {
                this.runTimeouts();
            }
        };
    }
    onCreated() {
        this.jobWrapperQueueMutex = new async_mutex_1.Mutex();
        this.runTimeouts();
        if (typeof document != "undefined") {
            document.addEventListener("visibilitychange", this.onVisibilityChanged);
        }
    }
    runTimeouts() {
        this.jobWrapperTimeout = setTimeout(async () => {
            return this.getJobWrappersFromQueue();
        }, 1000);
    }
    async getJobWrappersFromQueue() {
        const release = await this.jobWrapperQueueMutex.acquire();
        try {
            if (this.jobWrapperQueue.length > 0) {
                const groupByFilter = _.groupBy(this.jobWrapperQueue, (jobWrapperRequest) => {
                    return JSON.stringify(jobWrapperRequest.params.filters);
                });
                const processing = _.map(groupByFilter, async (jobWrapperByFilter, filterKey) => {
                    const jobUuids = _.uniq(_.map(jobWrapperByFilter, (val) => {
                        return val.params.jobUuid;
                    }));
                    const jobWrappers = await this.getJobWrappersCompact({
                        jobUuids, filters: filterKey !== "undefined" ? JSON.parse(filterKey) : undefined
                    });
                    const responseMessage = jobWrappers.getMessage();
                    const getJobWrappersReply = jobWrappers.getResult();
                    for (let wrapper of getJobWrappersReply.wrappers) {
                        const queueItems = _.filter(jobWrapperByFilter, (val) => {
                            return val.params.jobUuid === wrapper.job;
                        });
                        for (let queueItem of queueItems) {
                            const replyMessage = new esbInterface_1.EsbMessage(responseMessage.getFrom(), responseMessage.getMessageType(), responseMessage.getTimeStamp(), responseMessage.getSequenceNumber(), responseMessage.getMessageName(), {
                                job: getJobWrappersReply.jobs[wrapper.job],
                                parent: getJobWrappersReply.jobs[wrapper.parent],
                                progenitor: getJobWrappersReply.jobs[wrapper.progenitor]
                            });
                            const replyResult = new esbInterface_1.EsbResult(replyMessage);
                            queueItem.resolve(replyResult);
                        }
                    }
                });
                await Promise.all(processing);
            }
        }
        finally {
            this.jobWrapperQueue = [];
            release();
            this.jobWrapperTimeout = setTimeout(async () => {
                return this.getJobWrappersFromQueue();
            }, 1000);
        }
    }
    async addToJobWrapperQueue(resolve, params) {
        const release = await this.jobWrapperQueueMutex.acquire();
        try {
            this.jobWrapperQueue.push({ resolve, params });
        }
        finally {
            release();
        }
    }
    getAndLock(params) {
        return this.send('getAndLock', params);
    }
    getJobsForState(params) {
        return this.send('getJobsForState', params);
    }
    getJobUuidsForState(params) {
        return this.send('getJobUuidsForState', params);
    }
    getJobsForStateCompact(params) {
        return this.send("getJobsForStateCompact", params, { timeoutSecs: 600 });
    }
    getProgenitor(params) {
        return this.send('getProgenitor', params);
    }
    getJob(params) {
        return this.send('getJob', params);
    }
    getParent(params) {
        return this.send('getParent', params);
    }
    getJobWrapper(params) {
        if (params.batch) {
            const getJobWrapperPromise = new Promise((resolve, reject) => {
                this.addToJobWrapperQueue(resolve, params);
            });
            return getJobWrapperPromise;
        }
        else {
            return this.send('getJobWrapper', params);
        }
    }
    getJobWrappers(params) {
        return this.send('getJobWrappers', params);
    }
    getJobWrappersCompact(params) {
        return this.send("getJobWrappersCompact", params, { timeoutSecs: 600 });
    }
    getJobByInternalRef(params) {
        return this.send('getJobByInternalRef', params);
    }
    async jobExistsByInternalRef(params) {
        let result;
        try {
            await this.send('getJobByInternalRef', params);
            result = true;
        }
        catch (ex) {
            result = false;
        }
        return result;
    }
    getJobByExternalRef(params) {
        return this.send('getJobByExternalRef', params);
    }
    async jobExistsByExternalRef(params) {
        let result;
        try {
            await this.send('getJobByExternalRef', params);
            result = true;
        }
        catch (ex) {
            result = false;
        }
        return result;
    }
    getJobsByQuery(params) {
        return this.send('getJobsByQuery', params);
    }
    getJobUuidsByQuery(params) {
        return this.send('queryJobUuids', params);
    }
    getStateInfo(params) {
        return this.send('getStateInfo', params);
    }
    getStateInfoForStateChange(params) {
        return this.send('getStateInfoForStateChange', params);
    }
    getNextSerialNumber(params) {
        return this.send('getNextSerialNumber', params);
    }
    getStateHistory(params) {
        return this.send('getStateHistory', params);
    }
    getStateDefinition(params) {
        return this.send('getStateDef', params);
    }
    getStateDefinitionsByWorkflow(params) {
        return this.send('getStateDefinitionsByWorkflow', params);
    }
    getWorkflows() {
        return this.send('getWorkflows');
    }
    getAllTags() {
        return this.send('getAllTags');
    }
    tagJob(params) {
        return this.send('tagJob', params);
    }
    untagJob(params) {
        return this.send('untagJob', params);
    }
    async addJobInfo(params) {
        return this.send('addJobInfo', params);
    }
    async updateJobInfo(params, mergeInfo) {
        // when merging info from a previous version, add new array values to existing arrays where appropriate
        function concatArrays(objValue, srcValue) {
            if (_.isArray(objValue)) {
                return _.uniqWith(objValue.concat(srcValue), _.isEqual);
            }
        }
        const job = (await this.getJob({ jobUuid: params.jobUuid })).getResult();
        try {
            const currentJobInfo = _(job.info)
                .filter((info) => params.subType === info.subType)
                .sortBy((info) => info.version)
                .last();
            const newJobInfo = {
                jobUuid: job.uuid,
                infoUuid: currentJobInfo && currentJobInfo.infoUuid ? currentJobInfo.infoUuid : uuid.v4(),
                infoType: params.infoType,
                subType: params.subType,
                userUuid: params.userUuid,
                info: !mergeInfo ? params.info : _.mergeWith(currentJobInfo.info, params.info, concatArrays),
            };
            await this.send('addJobInfo', newJobInfo);
        }
        catch (e) {
            return undefined;
        }
        return;
    }
    async updateStateInfo(params) {
        const newStateInfo = {
            stateUuid: params.stateUuid,
            infoType: params.infoType,
            subType: params.subType,
            info: params.info
        };
        await this.send('addStateInfo', newStateInfo);
        return;
    }
    addTagInstanceInfo(params) {
        return this.send('addTagInstanceInfo', params);
    }
    startStep(params) {
        return this.send('startStep', params);
    }
    startStepById(params) {
        return this.send('startStep', params);
    }
    restartStep(params) {
        return this.send('restartStep', params);
    }
    forceState(params) {
        return this.send('forceState', params);
    }
    createJob(params) {
        return this.send('createJob', params);
    }
    createJobs(params) {
        return this.send('createJobs', params);
    }
    changeCompleteBy(params) {
        return this.send('changeCompleteBy', params);
    }
    changePriority(params) {
        return this.send('changePriority', params);
    }
    async updateStateProgress(params) {
        return await this.send('updateStateProgress', params);
    }
    getJobInfo(params) {
        return this.send('getJobInfo', params);
    }
    getJobInfos(params) {
        return this.send('getJobInfos', params);
    }
    getJobInfoDescriptors(params) {
        return this.send('getJobInfoDescriptors', params);
    }
    getJobCountForState(params) {
        return this.send('getJobCountForState', params);
    }
}
exports.StateStoreSubscriber = StateStoreSubscriber;
class JobBuilder extends builder_1.CommonJobBuilder {
    constructor(stateStoreSubscriber, job) {
        super(job);
        this.stateStoreSubscriber = stateStoreSubscriber;
    }
    parent(job) {
        this._parentId = job.uuid;
        return this;
    }
    parentId(jobUuid) {
        this._parentId = jobUuid;
        return this;
    }
    async stateStoreCreateJob() {
        return await this.create();
    }
    async create() {
        const params = { job: this.job };
        if (this._parentId) {
            params.parentId = this._parentId;
        }
        return await this.stateStoreSubscriber.createJob(params);
    }
}
exports.JobBuilder = JobBuilder;
