"use strict";
/**
 * Created by spencerrodd on 14/07/2015.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrameCallForInterval = exports.TimeRange = exports.Position = exports.Interval = exports.TcFormat = void 0;
var TcFormat;
(function (TcFormat) {
    TcFormat[TcFormat["NDF24"] = 0] = "NDF24";
    TcFormat[TcFormat["NDF25"] = 1] = "NDF25";
    TcFormat[TcFormat["NDF50"] = 2] = "NDF50";
    TcFormat[TcFormat["NDF60"] = 3] = "NDF60";
    TcFormat[TcFormat["DF30"] = 4] = "DF30";
    TcFormat[TcFormat["DF60"] = 5] = "DF60";
    TcFormat[TcFormat["DF24"] = 6] = "DF24";
    TcFormat[TcFormat["HUNDREDTHS"] = 7] = "HUNDREDTHS";
})(TcFormat || (exports.TcFormat = TcFormat = {}));
class FrameConstants {
    static getFps(fmt) {
        var FPS = {
            "NDF24": 24,
            "NDF25": 25,
            "NDF50": 50,
            "NDF60": 60,
            "DF30": (30000 / 1001),
            "DF60": (60000 / 1001),
            "DF24": (24000 / 1001)
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return FPS[fmtst];
    }
    static getDropFrames(fmt) {
        var NDF = {
            "NDF24": 0,
            "NDF25": 0,
            "NDF50": 0,
            "NDF60": 0,
            "DF30": 2,
            "DF60": 4,
            "DF24": 2
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return NDF[fmtst];
    }
    static getFramesPerHour(fmt) {
        var FPH = {
            "NDF24": 86400,
            "NDF25": 90000,
            "NDF50": 180000,
            "NDF60": 216000,
            "DF30": 107892,
            "DF60": 215784,
            "DF24": 86314
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return FPH[fmtst];
    }
    static getFramesPerDay(fmt) {
        return FrameConstants.getFramesPerHour(fmt) * 24;
    }
    static getFramesPerTenMinutes(fmt) {
        var FPT = {
            "NDF24": 14400,
            "NDF25": 15000,
            "NDF50": 30000,
            "NDF60": 36000,
            "DF30": 17982,
            "DF60": 35964,
            "DF24": 14386
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return FPT[fmtst];
    }
    static getFramesPerMinute(fmt) {
        var FPM = {
            "NDF24": 1440,
            "NDF25": 1500,
            "NDF50": 3000,
            "NDF60": 3600,
            "DF30": 1798,
            "DF60": 3598,
            "DF24": 1439
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return FPM[fmtst];
    }
    static getIntFps(fmt) {
        var IPS = {
            "NDF24": 24,
            "NDF25": 25,
            "NDF50": 50,
            "NDF60": 60,
            "DF30": 30,
            "DF60": 60,
            "DF24": 24
        };
        var fmtst;
        fmtst = TcFormat[fmt];
        return IPS[fmtst];
    }
    static getMsPerFrame(fmt) {
        return 1000 / FrameConstants.getFps(fmt);
    }
}
class IntMath {
    static div(v1, v2) {
        return Math.floor(v1 / v2);
    }
    static mod(v1, v2) {
        return Math.floor(v1 % v2);
    }
    static divRound(v1, v2) {
        return Math.round(v1 / v2);
    }
}
class Interval {
    static IntervalZero(fmt) {
        return new Interval(fmt, 0);
    }
    static IntervalFromSec(fmt, sec) {
        return new Interval(fmt, sec * 1000);
    }
    static IntervalFromFrames(fmt, fc) {
        return new Interval(fmt, fc * FrameConstants.getMsPerFrame(fmt));
    }
    static IntervalFromTcString(fmt, tc) {
        var result;
        var hh = parseInt(tc.substr(0, 2));
        var mm = parseInt(tc.substr(3, 2));
        var ss = parseInt(tc.substr(6, 2));
        var ff = parseInt(tc.substr(9, 2));
        switch (fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    var fps = FrameConstants.getFps(fmt);
                    result = new Interval(fmt, ((hh * 60 * 60 * fps) + (mm * 60 * fps) + (ss * fps) + ff) * 40);
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    var dropFrames = FrameConstants.getDropFrames(fmt);
                    var ifps = FrameConstants.getIntFps(fmt);
                    var tm = (60 * hh) + mm;
                    var fn = (ifps * 3600 * hh) + (ifps * 60 * mm) + (ifps * ss) + ff - (dropFrames * (tm - IntMath.div(tm, 10)));
                    result = Interval.IntervalFromFrames(fmt, fn);
                }
                break;
            case TcFormat.HUNDREDTHS:
                {
                    result = new Interval(fmt, ((hh * 60 * 60 * 1000) + (mm * 60 * 1000) + (ss * 1000) + (ff * 10)));
                }
                break;
        }
        return result;
    }
    constructor(fmt, millisec) {
        this.fmt = fmt;
        this.millisec = millisec;
        if (millisec < 0) {
            console.log("negative interval !!");
        }
    }
    pad(val, size) {
        var s = String(val);
        while (s.length < (size || 2)) {
            s = "0" + s;
        }
        return s;
    }
    getAsTcString() {
        var result;
        var hh;
        var mm;
        var ss;
        var ff;
        var fc = this.getAsFrame();
        switch (this.fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    var fps = FrameConstants.getFps(this.fmt);
                    hh = IntMath.mod(IntMath.div(fc, (60 * 60 * fps)), 24);
                    mm = IntMath.mod(IntMath.div(fc, (60 * fps)), 60);
                    ss = IntMath.mod(IntMath.div(fc, fps), 60);
                    ff = IntMath.mod(fc, fps);
                    result = this.pad(hh, 2) + ":" + this.pad(mm, 2) + ":" + this.pad(ss, 2) + ":" + this.pad(ff, 2);
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    var dropFrames = FrameConstants.getDropFrames(this.fmt);
                    var framesPerTenMinutes = FrameConstants.getFramesPerTenMinutes(this.fmt);
                    var ifps = FrameConstants.getIntFps(this.fmt);
                    var d = IntMath.div(fc, framesPerTenMinutes);
                    var m = IntMath.mod(fc, framesPerTenMinutes);
                    if (m > dropFrames) {
                        var fpm = FrameConstants.getFramesPerMinute(this.fmt);
                        fc += (dropFrames * 9 * d) + (dropFrames * IntMath.div((m - dropFrames), fpm));
                    }
                    else {
                        fc += dropFrames * 9 * d;
                    }
                    ff = IntMath.mod(fc, ifps);
                    var s = IntMath.div(fc, ifps);
                    ss = IntMath.mod(s, 60);
                    var m = IntMath.div(s, 60);
                    mm = IntMath.mod(m, 60);
                    hh = IntMath.div(m, 60);
                    result = this.pad(hh, 2) + ":" + this.pad(mm, 2) + ":" + this.pad(ss, 2) + ";" + this.pad(ff, 2);
                }
                break;
        }
        return result;
    }
    getAsMilliseconds() {
        return this.millisec;
    }
    getAsSeconds() {
        return this.millisec / 1000;
    }
    getAsFrame() {
        var result;
        switch (this.fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    result = IntMath.div(this.millisec, FrameConstants.getMsPerFrame(this.fmt));
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    result = IntMath.divRound(this.millisec, FrameConstants.getMsPerFrame(this.fmt));
                }
                break;
        }
        return result;
    }
    getFrameOfSecond() {
        return IntMath.mod(this.getAsFrame(), FrameConstants.getFps(this.fmt));
    }
    getAsMs() {
        return this.millisec;
    }
    add(other) {
        return new Interval(this.fmt, this.millisec + other.millisec);
    }
    subtract(other) {
        return new Interval(this.fmt, this.millisec - other.millisec);
    }
    nextFrame() {
        // add one frame interval to this interval
        return Interval.IntervalFromFrames(this.fmt, this.getAsFrame() + 1);
    }
    addFrames(frames) {
        return Interval.IntervalFromFrames(this.fmt, this.getAsFrame() + frames);
    }
    subtractFrames(frames) {
        return Interval.IntervalFromFrames(this.fmt, this.getAsFrame() - frames);
    }
    greaterThan(other) {
        return this.millisec > other.millisec;
    }
    lessThan(other) {
        return this.millisec < other.millisec;
    }
    correctedToCentre() {
        var result;
        var fc;
        var ms;
        fc = this.getAsFrame();
        ms = Math.floor(fc * FrameConstants.getMsPerFrame(this.fmt) + (FrameConstants.getMsPerFrame(this.fmt) / 2));
        return new Interval(this.fmt, ms);
    }
    equals(other) {
        return (this.correctedToCentre().getAsMs() === other.correctedToCentre().getAsMs());
    }
    getFormat() {
        return this.fmt;
    }
    isZero() {
        return this.millisec === 0;
    }
    floorSecond() {
        let sec = IntMath.div(this.millisec, 1000);
        return sec;
    }
}
exports.Interval = Interval;
class Position {
    static PositionZero(fmt) {
        return new Position(fmt, 0);
    }
    static PositionFromSec(fmt, sec) {
        return new Position(fmt, sec * 1000);
    }
    static PositionFromMs(fmt, ms) {
        return new Position(fmt, ms);
    }
    static PositionFromFrames(fmt, fc) {
        return new Position(fmt, fc * FrameConstants.getMsPerFrame(fmt));
    }
    static PositionFromTcString(fmt, tc) {
        var result;
        var hh = parseInt(tc.substr(0, 2));
        var mm = parseInt(tc.substr(3, 2));
        var ss = parseInt(tc.substr(6, 2));
        var ff = parseInt(tc.substr(9, 2));
        switch (fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    var fps = FrameConstants.getFps(fmt);
                    result = new Position(fmt, ((hh * 60 * 60 * fps) + (mm * 60 * fps) + (ss * fps) + ff) * (1000 / fps));
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    var dropFrames = FrameConstants.getDropFrames(fmt);
                    var ifps = FrameConstants.getIntFps(fmt);
                    var tm = (60 * hh) + mm;
                    var fn = (ifps * 3600 * hh) + (ifps * 60 * mm) + (ifps * ss) + ff - (dropFrames * (tm - IntMath.div(tm, 10)));
                    result = Position.PositionFromFrames(fmt, fn);
                }
                break;
        }
        return result;
    }
    constructor(fmt, millisec) {
        this.fmt = fmt;
        this.millisec = millisec;
        this.msInDay = (24 * 60 * 60 * 1000);
    }
    pad(val, size) {
        var s = String(val);
        while (s.length < (size || 2)) {
            s = "0" + s;
        }
        return s;
    }
    getAsTcString() {
        var result;
        var hh;
        var mm;
        var ss;
        var ff;
        var fc = this.getAsFrame();
        switch (this.fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    var fps = FrameConstants.getFps(this.fmt);
                    hh = IntMath.mod(IntMath.div(fc, (60 * 60 * fps)), 24);
                    mm = IntMath.mod(IntMath.div(fc, (60 * fps)), 60);
                    ss = IntMath.mod(IntMath.div(fc, fps), 60);
                    ff = IntMath.mod(fc, fps);
                    result = this.pad(hh, 2) + ":" + this.pad(mm, 2) + ":" + this.pad(ss, 2) + ":" + this.pad(ff, 2);
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    var dropFrames = FrameConstants.getDropFrames(this.fmt);
                    var framesPerTenMinutes = FrameConstants.getFramesPerTenMinutes(this.fmt);
                    var ifps = FrameConstants.getIntFps(this.fmt);
                    var d = IntMath.div(fc, framesPerTenMinutes);
                    var m = IntMath.mod(fc, framesPerTenMinutes);
                    if (m > dropFrames) {
                        fc += (dropFrames * 9 * d) + (dropFrames * IntMath.div((m - dropFrames), FrameConstants.getFramesPerMinute(this.fmt)));
                    }
                    else {
                        fc += dropFrames * 9 * d;
                    }
                    ff = IntMath.mod(fc, ifps);
                    var s = IntMath.div(fc, ifps);
                    ss = IntMath.mod(s, 60);
                    var m = IntMath.div(s, 60);
                    mm = IntMath.mod(m, 60);
                    hh = IntMath.div(m, 60);
                    result = this.pad(hh, 2) + ":" + this.pad(mm, 2) + ":" + this.pad(ss, 2) + ";" + this.pad(ff, 2);
                }
                break;
        }
        return result;
    }
    getAsSeconds() {
        return this.millisec / 1000;
    }
    getAsMilliseconds() {
        return this.millisec;
    }
    getAsFrame() {
        var result;
        switch (this.fmt) {
            case TcFormat.NDF25:
            case TcFormat.NDF50:
            case TcFormat.NDF60:
            case TcFormat.NDF24:
                {
                    result = IntMath.div(this.millisec, FrameConstants.getMsPerFrame(this.fmt));
                }
                break;
            case TcFormat.DF24:
            case TcFormat.DF30:
            case TcFormat.DF60:
                {
                    result = IntMath.divRound(this.millisec, FrameConstants.getMsPerFrame(this.fmt));
                }
                break;
        }
        return result;
    }
    subtractPosition(other) {
        return new Interval(this.fmt, this.millisec - other.millisec);
    }
    addInterval(other) {
        return new Position(this.fmt, this.millisec + other.getAsMs());
    }
    subtractInterval(other) {
        return new Position(this.fmt, this.millisec - other.getAsMs());
    }
    getFormat() {
        return this.fmt;
    }
    compareTo(other) {
        var result;
        result = this.millisec - other.millisec;
        // assume all events are less than 12 hours so 22:00 is < 02:00
        if (Math.abs(result) > (this.msInDay / 2)) {
            result = result * -1;
        }
        return result;
    }
    equals(other) {
        return this.getAsFrame() === other.getAsFrame();
    }
}
exports.Position = Position;
class TimeRange {
    constructor(start, len) {
        this.start = start;
        this.len = len;
    }
    getStart() {
        return this.start;
    }
    getLength() {
        return this.len;
    }
}
exports.TimeRange = TimeRange;
class FrameCallForInterval {
    constructor(intv, frameTickObj) {
        this.intv = intv;
        this.frameTickObj = frameTickObj;
    }
    updateTo(newintv) {
        var next;
        next = this.intv.nextFrame();
        while (newintv.greaterThan(next)) {
            this.frameTickObj.frameTick(next);
            this.intv = next;
            next = this.intv.nextFrame();
        }
    }
}
exports.FrameCallForInterval = FrameCallForInterval;
