"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJob = exports.getGlobal = exports.setGlobal = exports.log = exports.error = exports.getFieldFromObject = exports.getChildFieldWhere = exports.getPreviousInstance = exports.getChildValue = exports.getInputAtChildren = exports.getInputAt = exports.getInputOffset = exports.getIndexListWhere = exports.getInputWhere = void 0;
const _ = require("lodash");
const Log_1 = require("../logging/Log");
const useMock = typeof window !== 'undefined';
function getInputWhere(resultField, selector, where, test, comparator) {
    let loopInc = 1;
    let pos = this.transformState.position - 1;
    if (selector === 'FINDPREV') {
        loopInc = -1;
    }
    let ev;
    let found = false;
    do {
        pos = pos + loopInc;
        ev = this.transformState.inputValues[pos];
        if (!_.isUndefined(ev)) {
            const whereValue = ev[where];
            if (whereValue) {
                switch (test) {
                    case 'EQUALS':
                        found = whereValue === comparator;
                        break;
                    case 'NOTEQUALS':
                        found = whereValue !== comparator;
                        break;
                    case 'CONTAINS':
                        found = whereValue.match(comparator);
                        break;
                    case 'STARTSWITH':
                        found = whereValue.indexOf(comparator) === 0;
                        break;
                    case 'ENDSWITH':
                        found = whereValue.match(comparator + '$');
                        break;
                    case 'GREATER':
                        found = whereValue > comparator;
                        break;
                    case 'LESS':
                        found = whereValue < comparator;
                        break;
                    case 'MATCHES':
                        found = whereValue.match(comparator);
                        break;
                }
            }
        }
    } while (!_.isUndefined(ev) && !found);
    if (found) {
        return ev[resultField];
    }
    else {
        return '';
    }
}
exports.getInputWhere = getInputWhere;
function getIndexListWhere(where, test, comparator) {
    let ev;
    let arr = [];
    let found = false;
    for (var i = 0; i < this.transformState.metadata.itemCount; i++) {
        ev = this.transformState.inputValues[i];
        if (!_.isUndefined(ev)) {
            const whereValue = ev[where];
            if (whereValue) {
                switch (test) {
                    case 'EQUALS':
                        found = whereValue === comparator;
                        break;
                    case 'NOTEQUALS':
                        found = whereValue !== comparator;
                        break;
                    case 'CONTAINS':
                        found = whereValue.match(comparator);
                        break;
                    case 'STARTSWITH':
                        found = whereValue.indexOf(comparator) === 0;
                        break;
                    case 'ENDSWITH':
                        found = whereValue.match(comparator + '$');
                        break;
                    case 'GREATER':
                        found = whereValue > comparator;
                        break;
                    case 'LESS':
                        found = whereValue < comparator;
                        break;
                    case 'MATCHES':
                        found = whereValue.match(comparator);
                        break;
                }
            }
        }
        found && arr.push(i + 1);
    }
    return arr;
}
exports.getIndexListWhere = getIndexListWhere;
function getInputOffset(resultField, offset) {
    const pos = this.transformState.position + offset;
    const ev = this.transformState.inputValues[pos - 1];
    if (ev) {
        return ev[resultField];
    }
    else {
        return '';
    }
}
exports.getInputOffset = getInputOffset;
function getInputAt(resultField, offset, whereField, equals, mock) {
    if (this.onGetEventsAt) {
        if (useMock) {
            return mock;
        }
        else {
            const inputValues = this.onGetEventsAt({ offset });
            return _.find(inputValues, (e) => { return e[whereField] === equals; })?.[resultField];
        }
    }
    return mock;
}
exports.getInputAt = getInputAt;
function getInputAtChildren(offset, whereField, equals, mock) {
    if (this.onGetEventsAt) {
        if (useMock) {
            return mock;
        }
        else {
            const inputValues = this.onGetEventsAt({ offset });
            return getChildren(_.find(inputValues, (e) => { return e[whereField] === equals; }));
        }
    }
    return mock;
}
exports.getInputAtChildren = getInputAtChildren;
function getGrandchildren(input) {
    const childEvents = _.pickBy(input, (v, k) => {
        return k.startsWith("events");
    });
    const children = _.reduce(childEvents, (res, v, k) => {
        const id = Number(k.split(".")?.[1]);
        const value = { [_.slice(k.split("."), 2).join(".")]: v };
        res[id] = res[id] ? _.merge(res[id], value) : value;
        return res;
    }, []);
    return children;
}
function getGreatGrandchildren(input) {
    const childEvents = _.pickBy(input, (v, k) => {
        return k.match(/^events\.\d\.events/);
    });
    const children = _.reduce(childEvents, (res, v, k) => {
        const id = Number(k.split(".")?.[3]);
        const value = { [_.slice(k.split("."), 4).join(".")]: v };
        res[id] = res[id] ? _.merge(res[id], value) : value;
        return res;
    }, []);
    return children;
}
function getChildren(input) {
    let children = [];
    if (input) {
        const childEvents = _.pickBy(input, (v, k) => {
            return k.startsWith("children");
        });
        children = _.reduce(childEvents, (res, v, k) => {
            const id = Number(k.split(".")?.[1]);
            const value = { [_.slice(k.split("."), 2).join(".")]: v };
            res[id] = res[id] ? _.merge(res[id], value) : value;
            return res;
        }, []);
    }
    if (children) {
        const grandchildren = _.flatMap(children, child => {
            return getGrandchildren(child);
        }) || [];
        const greatGrandchildren = _.flatMap(children, child => {
            return getGreatGrandchildren(child);
        }) || [];
        return children.concat(grandchildren).concat(greatGrandchildren);
    }
}
function getChildValue(offset) {
    if (!offset) {
        offset = 0;
    }
    const input = this.transformState.inputValues?.[this.transformState.position - 1 + offset];
    return getChildren(input);
}
exports.getChildValue = getChildValue;
function getPreviousInstance(resultField, lookback, whereField, equals, mock) {
    if (this.this.onGetPreviousInputs) {
        if (useMock) {
            return mock;
        }
        else {
            return this.onGetPreviousInputs({ resultField, lookback, whereField, equals, mock });
        }
    }
    return mock;
}
exports.getPreviousInstance = getPreviousInstance;
function getChildFieldWhere(path, array, field, test, value) {
    const input = this.transformState.inputValues[this.transformState.position - 1];
    const foundField = _.find(input[array], (element) => {
        let found = false;
        switch (test) {
            case 'EQUALS':
                found = element[field] === value;
                break;
            case 'NOTEQUALS':
                found = element[field] !== value;
                break;
            case 'CONTAINS':
                found = !!element[field].match(value);
                break;
            case 'STARTSWITH':
                found = element[field].indexOf(value) === 0;
                break;
            case 'ENDSWITH':
                found = !!element[field].match(value + '$');
                break;
            case 'GREATER':
                found = element[field] > value;
                break;
            case 'LESS':
                found = element[field] < value;
                break;
            case 'MATCHES':
                found = !!element[field].match(value);
                break;
        }
        return found;
    });
    if (foundField && _.get(foundField, path)) {
        return _.get(foundField, path);
    }
    else {
        return '';
    }
}
exports.getChildFieldWhere = getChildFieldWhere;
function getFieldFromObject(object, field) {
    return _.isUndefined(_.get(object, [field])) ? '' : _.get(object, [field]);
}
exports.getFieldFromObject = getFieldFromObject;
function error(message, description) {
    this.transformState.errors.push({ message, description, input: this.transformState.inputValues[this.transformState.position - 1] });
}
exports.error = error;
function log(level, message) {
    this.transformState.log.push({ level, message });
    if (Log_1.Log.getLogLevel(Log_1.LogLevel.TRACE)) {
        Log_1.Log[level](message + "\t" + `[ position: ${this.transformState.position}, globals: ${JSON.stringify(_.pick(this.transformState, ['globals', 'values']))}, currentRow: ${JSON.stringify(this.transformState.inputValues[this.transformState.position - 1])} ]`);
    }
    else {
        Log_1.Log[level](message);
    }
}
exports.log = log;
function setGlobal(varName, varVal) {
    this.transformState.globals[varName] = varVal;
}
exports.setGlobal = setGlobal;
function getGlobal(varName) {
    return this.transformState.globals[varName];
}
exports.getGlobal = getGlobal;
function getJob() {
}
exports.getJob = getJob;
