"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunRuleResult = void 0;
const _ = require("lodash");
const Log_1 = require("../logging/Log");
class RunRuleResult {
    constructor(alarmSubscriber, stateStoreSubscriber) {
        this.alarmSubscriber = alarmSubscriber;
        this.stateStoreSubscriber = stateStoreSubscriber;
    }
    async run(transformResult) {
        try {
            await this.makeJobs(transformResult);
            await this.triggerAlarms(transformResult);
            await this.clearAlarms(transformResult);
        }
        catch (e) {
            console.error(e);
        }
    }
    async makeJobs(transformResult) {
        if (transformResult.parentJob) {
            const parentJob = transformResult.parentJob;
            parentJob.children = transformResult.jobs;
            return await this.stateStoreSubscriber.createJob({ job: parentJob });
        }
        else if (!_.isEmpty(transformResult.jobs)) {
            return await this.stateStoreSubscriber.createJobs({ jobs: transformResult.jobs });
        }
    }
    async triggerAlarms(transformResult) {
        for (const alarm of transformResult.alarms) {
            try {
                await this.alarmSubscriber.triggerAlarm(alarm.id, true, alarm.description, alarm.description);
            }
            catch (e) {
                Log_1.Log.logError("Could not trigger alarm " + e.message);
            }
        }
        return true;
    }
    async clearAlarms(transformResult) {
        for (const alarm of transformResult.clearAlarms) {
            try {
                await this.alarmSubscriber.triggerAlarm(alarm.id, false, alarm.description);
            }
            catch (e) {
                Log_1.Log.logError("Could not clear alarm " + e.message);
            }
        }
        return true;
    }
}
exports.RunRuleResult = RunRuleResult;
