"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const node_1 = require("@busby/esb/node");
const shell_1 = require("./shell");
const usage = () => {
    commander_1.program.outputHelp();
    process.exit(1);
};
const fs = require("fs-extra");
const login_1 = require("./login");
let defaultRabbit = "localhost";
if (process.env.RABBIT) {
    defaultRabbit = process.env.RABBIT;
}
else if (fs.existsSync("/opt/squaredpaper/busby.conf")) {
    try {
        const conf = fs.readJSONSync("/opt/squaredpaper/busby.conf");
        defaultRabbit = conf.corehost;
    }
    catch { }
}
commander_1.program
    .version('1.0.0')
    .showHelpAfterError(true)
    .option('-r, --rabbit <rabbit>', 'Host of the RabbitMQ cluster', defaultRabbit)
    .option('-u, --user <user>', 'Username to log in to RabbitMQ cluster', 'squared')
    .option('-p, --password <password>', 'Password for the RabbitMQ cluster', 'Squared1')
    .option('-s, --shell <shell>', 'The shell to run')
    .argument("<host>", "The hostname to connect to, the name that is used by the busby monitor is required");
commander_1.program.parse(process.argv);
const options = commander_1.program.opts();
const shell = options.shell;
const hostName = commander_1.program.args[0];
const args = shell ? process.argv.slice(process.argv.indexOf(shell) + 1) : process.argv.slice(process.argv.indexOf(hostName) + 1);
if (!hostName) {
    usage();
}
new node_1.LoginDetails(options.rabbit, options.user, options.password, "esbShell");
node_1.Log.setFileName("esbShell");
node_1.Log.setLogLevel(node_1.LogLevel.TRACE, true);
async function main() {
    const params = {};
    await (0, login_1.credentials)(params);
    const authentication = node_1.AuthenticationServiceSubscriber.get("authenticationService");
    const auth = await authentication.authenticateUser({
        username: params.user,
        password: params.password,
        source: "esbShell",
    });
    const permissions = await authentication.getUserPermissionsFromSessionId({ sessionId: auth.getResult().sessionId });
    if (!permissions.getResult().busbyAdmin?.permissions?.includes("canAccessShell")) {
        console.error("You do not have permissions to connect to a shell");
        process.exit(3);
    }
    (0, shell_1.openShell)(hostName, args, shell);
}
main();
//# sourceMappingURL=esbShell.js.map