"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("@busby/esb/node");
const commander_1 = require("commander");
const luxon_1 = require("luxon");
const colorize = require("json-colorizer");
const Chance = require("chance");
const repl = require("repl");
const events_1 = require("events");
const util = require("util");
const fs = require("fs-extra");
const chance = new Chance();
const usage = async () => {
    commander_1.program.outputHelp();
    await new Promise(() => { });
    process.exit(1);
};
let defaultRabbit = "localhost";
if (process.env.RABBIT) {
    defaultRabbit = process.env.RABBIT;
}
else if (fs.existsSync("/opt/squaredpaper/busby.conf")) {
    try {
        const conf = fs.readJSONSync("/opt/squaredpaper/busby.conf");
        defaultRabbit = conf.corehost;
    }
    catch { }
}
const readFile = util.promisify(require("fs").readFile);
commander_1.program
    .version('1.0.0')
    .showHelpAfterError(true)
    .option('-r, --rabbit <user>', 'Host of the RabbitMQ cluster', defaultRabbit)
    .option('-u, --user <user>', 'Username to log in to RabbitMQ cluster', 'squared')
    .option('-p, --password <password>', 'Password for the RabbitMQ cluster', 'Squared1')
    .option('-c, --cli', 'cli', false)
    .argument('<service>', 'The service to connect to on the bus');
commander_1.program.parse(process.argv);
const options = commander_1.program.opts();
var serviceName = commander_1.program.args[0];
if (!serviceName) {
    usage();
}
console.log(`Connecting to ${options['rabbit']} - (${options['user']}:${options['password']}) ${serviceName}`);
let mode = options['cli'] ? 'cli' : 'bus';
main();
var emitter = new events_1.EventEmitter();
var showStatus = false;
var filterMessageNames = [];
var filterMode = "include";
var emitStatus = false;
var showStatusDs = false;
var timing = false;
var sub;
var subscriber;
async function main() {
    node_1.Log.setLogLevel(node_1.LogLevel.TRACE, true);
    node_1.Log.setFileName("esbTestSubscriber");
    new node_1.LoginDetails(options['rabbit'], options['user'], options['password'], "esbTestSubscriber");
    subscriber = node_1.GenericSubscriber.get(serviceName, undefined, { promiscuous: true });
    sub = subscriber.getSubscriber();
    sub.setDirectoryStatusProcessor(dirStatusProcessor);
    subscriber.waitUntilConnected()
        .then((connected) => {
        if (connected) {
            console.log('Connected to service');
            startRepl();
        }
        else {
            console.error('Could not connect');
            process.exit();
        }
    }).catch((err) => {
        console.error(err);
    });
}
function processHelp() {
    console.log('status(boolean, messageName?) - show or hide status events, optional message name');
    console.log('statusDs(boolean) - show or hide status events for the directory');
    console.log('cli(command) - send a cli command to a service - if command is true you will enter cli mode');
    console.log('cliFile(filename) - send a file of cli commands to a service');
    console.log('emitStatus(boolean) - enable or disable emitting of status events');
    console.log('send(command, params) - send the command');
    console.log('sendDs(command, params) - send the command to the directory service');
    console.log('sendPromise(command, params) - send the command and get a Promise back to perform .then()');
    console.log('sendfile(filename) - send a file of commands');
    console.log('timing(boolean) - show time taken for command instad of result');
    console.log('emitter - where status events are emitted from, Event = Message Name, Arguments = (Message Params)');
    console.log('spam(command, params, repeat, pause?) - send the command as fast as possible repeat times');
    console.log('esbLog()');
    console.log(`filter(messageNames, mode) - filter for status message names e.g. filter(["jobCreated"], "exclude") will exclude messages named jobCreated`);
    return 0;
}
function processStatus(val, messageName = '#') {
    showStatus = val;
    const s = subscriber.status[messageName](statusProcessor);
    if (!val) {
        s.dispose();
    }
    return 'show status ' + showStatus + ' ' + messageName;
}
function processEmitStatus(val) {
    emitStatus = val;
    const s = subscriber.status['#'](statusEmitter);
    if (!val) {
        s.dispose();
    }
    return 'emit status ' + emitStatus;
}
function processStatusDs(val) {
    showStatusDs = val;
    return 'show status ds ' + showStatusDs;
}
function processTiming(val) {
    timing = val;
    return 'timing ' + true;
}
function processSetupFilters(val, mode) {
    if (!val) {
        filterMessageNames = [];
        return 'filters off';
    }
    filterMessageNames = val;
    filterMode = mode || "include";
    if (val.length > 0) {
        return `filter to ${filterMode} message names ${val.join(", ")}`;
    }
    else {
        return 'no filters defined showing all messages';
    }
}
async function sleep(ms) {
    return new Promise((resolve) => {
        setTimeout(() => {
            resolve(ms);
        }, ms);
    });
}
async function processSpam(command, params, repeat, pause) {
    for (let i = 0; i < repeat - 1; i++) {
        const reply = await sub.sendControl(command, { ...params, i });
        console.log(i + ": " + JSON.stringify(reply.getAsJson()));
        if (pause) {
            await sleep(pause);
        }
    }
}
function processSend(command, params, options) {
    const startTime = luxon_1.DateTime.utc();
    sub.sendControl(command, params)
        .then((reply) => {
        if (reply.getAsJson() && reply.getAsJson().params && reply.getAsJson().params.status === 'error') {
            console.error(JSON.stringify(reply.getAsJson()));
        }
        else {
            if (timing) {
                console.log(startTime.diffNow().toFormat("hh:mm:ss.SSS"));
            }
            else {
                console.log(options?.pretty ? colorize(reply.getAsJson(), { pretty: true }) : JSON.stringify(reply.getAsJson()));
            }
        }
    })
        .catch(err => {
        console.error(err);
    });
    return 'Sending ' + command + '...';
}
function processCli(command) {
    if (command === true) {
        enterCli();
        return null;
    }
    var commandLine = { "commandLine": command };
    sub.sendControl("qqCliCommand", commandLine)
        .then((reply) => {
        if (reply.getAsJson() && reply.getAsJson().params && reply.getAsJson().params.status === 'error') {
            console.error(JSON.stringify(reply.getAsJson()));
        }
        else {
            console.log("");
            for (const rl of reply.getAsJson().params.result) {
                console.log(rl);
            }
        }
    })
        .catch(err => {
        console.error(err);
    });
    return 'Sending cli command ' + command;
}
function processSendPromise(command, params) {
    return sub.sendControl(command, params)
        .catch(err => {
        console.error(err);
    });
}
function processSendDs(command, params) {
    const startTime = luxon_1.DateTime.utc();
    sub.sendDs(command, params)
        .then((reply) => {
        if (reply.getAsJson() && reply.getAsJson().params && reply.getAsJson().params.status === 'error') {
            console.error(JSON.stringify(reply.getAsJson()));
        }
        else {
            if (timing) {
                console.log(luxon_1.DateTime.utc().diff(startTime).toFormat("hh:mm:ss.SSS"));
            }
            else {
                console.log(JSON.stringify(reply.getAsJson()));
            }
        }
    })
        .catch((err) => {
        console.error(err);
    });
    return 'Sending ' + command + '...';
}
function processCliFile(filename, options) {
    readFile(filename, 'utf-8')
        .then((fileContents) => {
        const lines = fileContents.split("\n");
        for (const line of lines) {
            console.log(line);
            processCli(line);
        }
    });
    return "done";
}
let bus;
let busEval;
async function customEval(evalCmd, context, file, cb) {
    const commandLine = evalCmd.replace(/[\r\n]*/g, "");
    if (mode === "bus" || commandLine === ".exit") {
        busEval.bind(bus)(evalCmd, context, file, cb);
    }
    else if (mode === "cli") {
        cb(null, await cliCommand(commandLine));
    }
}
function startRepl() {
    bus = repl.start({
        prompt: `${mode} (${serviceName}) >> `,
    });
    busEval = bus.eval;
    bus.eval = customEval;
    if (mode === "cli") {
        enterCli();
    }
    // command lookup table
    bus.context["help"] = processHelp;
    bus.context["timing"] = processTiming;
    bus.context["status"] = processStatus;
    bus.context["send"] = new Proxy(processSend, {
        get: (target, p) => {
            if (typeof p !== "string") {
                return target;
            }
            return (params, options) => target(p, params, options);
        }
    });
    bus.context["sendDs"] = processSendDs;
    bus.context["statusDs"] = processStatusDs;
    bus.context["filter"] = processSetupFilters;
    bus.context["sendPromise"] = new Proxy(processSendPromise, {
        get: (target, p) => {
            if (typeof p !== "string") {
                return target;
            }
            return (params) => target(p, params);
        }
    });
    ;
    bus.context["emitStatus"] = processEmitStatus;
    bus.context["cli"] = processCli;
    bus.context["cliFile"] = processCliFile;
    bus.context["emitter"] = emitter;
    bus.context["enterCli"] = enterCli;
    bus.context["spam"] = processSpam;
    bus.context["esbLog"] = esbLog;
    bus.context["subscriber"] = subscriber;
}
function esbLog() {
    node_1.Log.setLogLevel(node_1.LogLevel.ESB, true);
}
function enterCli() {
    mode = "cli";
    bus.ignoreUndefined = true;
    bus.setPrompt(`${mode} (${serviceName}) >> `);
}
async function cliCommand(commandLine) {
    if (commandLine === "exit") {
        mode = "bus";
        bus.ignoreUndefined = false;
        bus.setPrompt(`${mode} (${serviceName}) >> `);
        return;
    }
    const reply = await subscriber.control.qqCliCommand({ commandLine }).catch(err => { });
    if (!reply) {
        console.error(`ERROR NO RESPONSE`);
    }
    else if (reply.isError()) {
        console.error(`ERROR ${reply?.getErrorResult().reason}`);
    }
    else {
        const lines = reply.getResult().result;
        for (const line of lines) {
            if (line !== ">") {
                console.log(line);
            }
        }
    }
    if (commandLine === "?") {
        console.log("");
        console.log("exit         Returns to bus mode");
    }
    return;
}
function statusProcessor(esbMessage) {
    let doShow = true;
    if (filterMessageNames.length > 0) {
        if (filterMode === "include") {
            if (!filterMessageNames.includes(esbMessage.getMessageName())) {
                doShow = false;
            }
        }
        else if (filterMode === "exclude") {
            if (filterMessageNames.includes(esbMessage.getMessageName())) {
                doShow = false;
            }
        }
    }
    if (doShow) {
        if (showStatus) {
            if (!serviceName.startsWith('Monitor_') || esbMessage.getMessageName().startsWith("pty")) {
                console.log(JSON.stringify(esbMessage.getAsJson()));
            }
        }
    }
}
function statusEmitter(esbMessage) {
    if (emitStatus) {
        emitter.emit("name", esbMessage.getMessageName());
        emitter.emit(esbMessage.getMessageName(), esbMessage.getParams());
    }
}
function dirStatusProcessor(esbMessage) {
    let doShow = true;
    if (filterMessageNames.length > 0) {
        if (filterMode === "include") {
            if (!filterMessageNames.includes(esbMessage.getMessageName())) {
                doShow = false;
            }
        }
        else if (filterMode === "exclude") {
            if (filterMessageNames.includes(esbMessage.getMessageName())) {
                doShow = false;
            }
        }
    }
    if (showStatusDs && doShow) {
        console.log(JSON.stringify(esbMessage.getAsJson()));
    }
}
//# sourceMappingURL=esbSubscriber.js.map