"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.credentials = void 0;
const readline = require("readline");
const constants_1 = require("constants");
const credentials = async (options) => {
    let promise;
    let resolve;
    process.stdin.setEncoding("utf-8");
    options.user ?? (options.user = "");
    options.password ?? (options.password = "");
    const pn = (field) => (data, key) => {
        const c = data;
        switch (c) {
            case '\u0004': // Ctrl-d
            case '\r':
            case '\n':
                return enter(field);
            case '\u0003': // Ctrl-c
                return ctrlc(field);
            default:
                // backspace
                if (c.charCodeAt(0) === 8 || c.charCodeAt(0) === 127) {
                    return backspace(field);
                }
                else {
                    return newchar(field, c);
                }
        }
    };
    const listeners = {
        user: pn("user"),
        password: pn("password"),
    };
    function enter(field) {
        process.stdin.removeListener('data', listeners[field]);
        process.stdin.setRawMode(false);
        process.stdin.pause();
        resolve();
    }
    function ctrlc(field) {
        process.stdin.removeListener('data', listeners[field]);
        process.stdin.setRawMode(false);
        process.stdin.pause();
        process.exit(constants_1.SIGINT);
    }
    function newchar(field, c) {
        options[field] += c;
        process.stdout.write(field === "password" ? "*" : c);
    }
    function backspace(field) {
        const text = field === "password" ? "Password: " : "Username: ";
        readline.cursorTo(process.stdout, text.length + options[field].length - 1);
        process.stdout.write(" ");
        readline.moveCursor(process.stdout, -1, 0);
        options[field] = options[field].slice(0, options[field].length - 1);
    }
    if (!options.user) {
        promise = new Promise((res) => resolve = res);
        process.stdout.write("Username: ");
        process.stdin.resume();
        process.stdin.setRawMode(true);
        process.stdin.on("data", listeners["user"]);
        await promise;
    }
    process.stdout.write(`\n`);
    promise = new Promise((res) => resolve = res);
    process.stdout.write("Password: ");
    process.stdin.resume();
    process.stdin.setRawMode(true);
    process.stdin.on("data", listeners["password"]);
    await promise;
    process.stdout.write(`\n`);
};
exports.credentials = credentials;
//# sourceMappingURL=login.js.map