"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.openShell = void 0;
const esb_1 = require("@busby/esb");
const uuid_1 = require("uuid");
const openShell = async (hostName, args, shell) => {
    const cliSize = () => {
        return {
            cols: process.stdout.columns || 80, rows: process.stdout.rows || 60
        };
    };
    const cliResize = () => {
        process.on("SIGWINCH", onResize);
    };
    const onResize = () => {
        subscriber.control.ptyResize({
            id: ptyId,
            data: cliSize(),
        });
    };
    cliResize();
    const ptyId = (0, uuid_1.v4)();
    const subscriber = esb_1.MonitorSubscriber.get(`Monitor_${hostName}`);
    const onRemoteData = (data) => {
        const params = data.getParams();
        if (params.id === ptyId) {
            process.stdout.write(params.data);
        }
    };
    const onClose = (data) => {
        const params = data.getParams();
        if (params.id === ptyId) {
            console.error(` --- Remote session ended with code ${params.exit?.exitCode ?? 1} --- `);
            process.stdin.off('data', onLocalData);
            if (params.exit?.exitCode === 0) {
                process.exit(0);
            }
            else {
                // Allow some time for any data coming across
                setTimeout(() => process.exit(params.exit?.exitCode ?? 1), 2500);
            }
        }
    };
    const onLocalData = (data) => subscriber.control.ptyData({
        id: ptyId,
        data: data.toString(),
    });
    const onHeartbeat = async () => {
        await subscriber.control.ptyHeartbeat({
            id: ptyId,
        });
        setTimeout(onHeartbeat, 60 * 1000);
    };
    console.log(`Connecting to ${hostName}`);
    await subscriber.waitUntilConnected();
    const request = await subscriber.control.getSystemHealth();
    const health = request.getResult();
    subscriber.status.ptyData(onRemoteData);
    subscriber.status[`ptyData.${ptyId}`](onRemoteData);
    subscriber.status.ptyClose(onClose);
    const create = await subscriber.control.ptyStart({
        id: ptyId,
        shell: shell ? shell : health.osType === "Windows_NT" ? (health.bash ? "C:\\opt\\squaredpaper\\git\\bin\\bash.exe" : health.wsl ? "bash.exe" : "powershell.exe") : "bash",
        args,
    });
    if (create.isError()) {
        console.error("Unable to initiate session");
        process.exit(1);
    }
    setTimeout(onHeartbeat, 60 * 1000);
    console.log(`Connected to Session ${ptyId} for ${hostName}`);
    process.stdin.setRawMode(true);
    process.stdin.resume();
    process.stdin.on('data', onLocalData);
    onResize();
};
exports.openShell = openShell;
//# sourceMappingURL=shell.js.map