"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const selection_1 = require("./selection");
const logging_1 = require("./logging");
const configurations_1 = require("./configurations");
const extensionUpdater_1 = require("./updater/extensionUpdater");
const esb_1 = require("@busby/esb");
const terminals_1 = require("./terminals");
function activate(context) {
    (0, logging_1.setupLogging)();
    (0, extensionUpdater_1.setupUpdater)(context);
    for (let key of context.globalState.keys()) {
        const val = context.globalState.get(key);
        if (key.startsWith("busby.providers")) {
            const [, suffix] = key.split("busby.providers.");
            if (suffix.includes(".")) {
                context.globalState.update(key, undefined);
                key = `busby.providers.${suffix.replaceAll(".", "-")}`;
                context.globalState.update(key, val);
            }
        }
        if (key.startsWith("busby.nodes")) {
            const [, suffix] = key.split("busby.nodes.");
            if (suffix.includes(".")) {
                context.globalState.update(key, undefined);
                key = `busby.nodes.${suffix.replaceAll(".", "-")}`;
                context.globalState.update(key, val);
            }
        }
        if (key.startsWith("busby.services")) {
            const [, suffix] = key.split("busby.services.");
            if (suffix.includes(".")) {
                context.globalState.update(key, undefined);
                key = `busby.services.${suffix.replaceAll(".", "-")}`;
                context.globalState.update(key, val);
            }
        }
    }
    const disposable = vscode.Disposable.from(vscode.commands.registerCommand("squaredpaper.checkForExtensionUpdate", () => {
        (0, extensionUpdater_1.checkNewVersion)(context, true);
    }), vscode.commands.registerCommand("squaredpaper.reinstall", () => {
        (0, extensionUpdater_1.checkNewVersion)(context, true, true);
    }), vscode.commands.registerCommand("squaredpaper.checkState", async () => {
        esb_1.Log.logInfo(`Global State Keys: ${context.globalState.keys().length}`);
        for (let key of context.globalState.keys()) {
            const val = context.globalState.get(key);
            esb_1.Log.logInfo(`  ${key}:\n    ${JSON.stringify(val)}`);
        }
        esb_1.Log.logInfo(`Workspace State Keys: ${context.workspaceState.keys().length}`);
        for (let key of context.workspaceState.keys()) {
            const val = context.workspaceState.get(key);
            esb_1.Log.logInfo(`  ${key}:\n    ${JSON.stringify(val)}`);
        }
    }), vscode.commands.registerCommand('squaredpaper.pickProject', async () => {
        return await (0, selection_1.pickProject)(context);
    }), vscode.commands.registerCommand('squaredpaper.pickBusbyCore', async () => {
        return await (0, selection_1.pickBusbyCore)(context);
    }), vscode.commands.registerCommand('squaredpaper.pickBusbyUrl', async () => {
        return await (0, selection_1.pickBusbyUrl)(context);
    }), vscode.commands.registerCommand('squaredpaper.pickService', async () => {
        return await (0, selection_1.pickService)(context);
    }), vscode.commands.registerCommand('squaredpaper.pickProvider', async ({ connect = false }) => {
        return await (0, selection_1.pickProvider)(context, connect);
    }), vscode.commands.registerCommand('squaredpaper.pickBusbyNode', async () => {
        return await (0, selection_1.pickBusbyNode)(context);
    }), vscode.commands.registerCommand('squaredpaper.lastProject', async () => {
        const lastProject = context.workspaceState.get("lastProject", "");
        return lastProject;
    }), vscode.commands.registerCommand('squaredpaper.lastBusbyCore', async () => {
        const lastBusbyHost = context.workspaceState.get("lastBusbyHost", "");
        return lastBusbyHost;
    }), vscode.commands.registerCommand('squaredpaper.lastService', async () => {
        const lastService = context.workspaceState.get("lastService", "");
        return lastService;
    }), vscode.commands.registerCommand('squaredpaper.lastProvider', async () => {
        const lastProvider = context.workspaceState.get("lastProvider", "");
        return lastProvider;
    }), vscode.commands.registerCommand('squaredpaper.lastBusbyNode', async () => {
        const lastBusbyNode = context.workspaceState.get("lastNode", "");
        return lastBusbyNode;
    }), vscode.commands.registerCommand('squaredpaper.addLaunchConfig', async (folder) => {
        await (0, configurations_1.addLaunchConfiguration)(folder);
    }), vscode.commands.registerCommand('squaredpaper.clearHistory', async (folder) => {
        context.workspaceState.update("lastProject", "");
        context.workspaceState.update("lastServices", []);
        context.workspaceState.update("lastProviders", []);
        context.globalState.update("lastProjects", []);
    }), vscode.workspace.onDidChangeConfiguration((0, configurations_1.onConfigChanged)(context)), vscode.commands.registerCommand('squaredpaper.shell', async () => {
        (0, terminals_1.shell)(context);
    }), vscode.commands.registerCommand('squaredpaper.shellWs', async () => {
        (0, terminals_1.shellWs)(context);
    }), vscode.commands.registerCommand('squaredpaper.shellDirect', async () => {
        (0, terminals_1.shellDirect)(context);
    }), vscode.commands.registerCommand('squaredpaper.cli', async () => {
        (0, terminals_1.cli)(context);
    }), vscode.commands.registerCommand('squaredpaper.subscriber', async () => {
        (0, terminals_1.subscriber)(context);
    }), vscode.commands.registerCommand('squaredpaper.shellEditor', async () => {
        (0, terminals_1.shell)(context, vscode.TerminalLocation.Editor);
    }), vscode.commands.registerCommand('squaredpaper.shellWsEditor', async () => {
        (0, terminals_1.shellWs)(context, vscode.TerminalLocation.Editor);
    }), vscode.commands.registerCommand('squaredpaper.shellDirectEditor', async () => {
        (0, terminals_1.shellDirect)(context, vscode.TerminalLocation.Editor);
    }), vscode.commands.registerCommand('squaredpaper.cliEditor', async () => {
        (0, terminals_1.cli)(context, vscode.TerminalLocation.Editor);
    }), vscode.commands.registerCommand('squaredpaper.subscriberEditor', async () => {
        (0, terminals_1.subscriber)(context, vscode.TerminalLocation.Editor);
    }));
    context.subscriptions.push(disposable);
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map