"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateLogLevels = exports.setupLogging = void 0;
const esb_1 = require("@busby/esb");
const vscode = require("vscode");
function setupLogging() {
    esb_1.Log.disableFileLogging();
    const outputChannel = vscode.window.createOutputChannel("Squared Paper", "log");
    esb_1.Log.setLogLevel(esb_1.LogLevel.DEBUG, true);
    updateLogLevels();
    esb_1.Log.addLogDestination((le) => {
        let formattedMessage;
        if (le.getSubfile()) {
            formattedMessage = `[${le.getDate().toISO()}] [${le.getSubfile()}] [${esb_1.LogLevel[le.getLogLevel()]}]: ${le.getMessage()}`;
        }
        else {
            formattedMessage = `[${le.getDate().toISO()}] [main] [${esb_1.LogLevel[le.getLogLevel()]}]: ${le.getMessage()}`;
        }
        outputChannel.appendLine(formattedMessage);
    });
    process.on('uncaughtException', (err, origin) => {
        esb_1.Log.logError(`Uncaught Exception ${err.message}`, err);
    });
    process.on('unhandledRejection', (reason, promise) => {
        esb_1.Log.logError(`Unhandled Rejection ${reason}`, promise);
    });
}
exports.setupLogging = setupLogging;
function updateLogLevels() {
    const config = vscode.workspace.getConfiguration("squaredpaper").get("logging", {});
    for (const [key, state] of Object.entries(config)) {
        if (typeof state === "boolean") {
            esb_1.Log.setLogLevel(key, state);
        }
    }
}
exports.updateLogLevels = updateLogLevels;
//# sourceMappingURL=logging.js.map