"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickBusbyCore = void 0;
const esb_1 = require("@busby/esb");
const vscode = require("vscode");
const vscode_1 = require("vscode");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickBusbyCore(context) {
    let result = undefined;
    const quickPick = vscode_1.window.createQuickPick();
    quickPick.ignoreFocusOut = true;
    const hosts = [];
    const lastHost = context.workspaceState.get("lastBusbyHost", "");
    if (lastHost) {
        hosts.push({
            label: "Last Host in Workspace",
            kind: vscode_1.QuickPickItemKind.Separator
        }, {
            label: lastHost,
            picked: true,
        });
        quickPick.selectedItems = [hosts[1]];
    }
    const lastHosts = context.globalState.get("lastBusbyHosts", []);
    if (lastHosts.length > 0) {
        for (const host of lastHosts) {
            if (host === lastHost) {
                continue;
            }
            const item = {
                label: host,
                picked: !lastHost && lastHosts.indexOf(host) === 0,
                buttons: [
                    {
                        iconPath: new vscode.ThemeIcon('close'),
                        tooltip: "Remove",
                    }
                ]
            };
            hosts.push(item);
            if (!quickPick.selectedItems && !lastHost && lastHosts.indexOf(host) === 0) {
                quickPick.selectedItems = [item];
            }
        }
    }
    quickPick.items = [...hosts];
    quickPick.title = "Select or enter a host";
    result = await new Promise((resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidChangeValue((value) => {
            if (!hosts.find(h => h.label === value)) {
                quickPick.items = [{ label: quickPick.value, description: "New Host", }, ...hosts];
            }
            else {
                quickPick.items = hosts;
            }
        }), quickPick.onDidTriggerItemButton((e) => {
            hosts.splice(hosts.indexOf(e.item), 1);
            quickPick.items = hosts;
            context.globalState.update("lastBusbyHosts", hosts.filter(h => h.description !== "New Host" && h.kind !== vscode_1.QuickPickItemKind.Separator).map(h => h.label));
        }), quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            resolve(selection);
            disposable.dispose();
        }));
        quickPick.show();
    });
    const chosen = result?.label;
    if (chosen) {
        const lastHosts = context.globalState.get("lastBusbyHosts", []);
        if (lastHosts.includes(chosen)) {
            lastHosts.splice(lastHosts.indexOf(chosen), 1);
        }
        lastHosts.unshift(chosen);
        const config = vscode.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("core") ?? 0;
        if (history > 0) {
            if (lastHosts.length > history) {
                lastHosts.splice(history - 1, lastHosts.length - history);
            }
        }
        esb_1.Log.logInfo(`Last Host Changed to ${chosen}`);
        esb_1.Log.logInfo(`Last Hosts Changed to ${lastHosts.length} ${lastHosts.join(", ")}`);
        context.globalState.update("lastBusbyHosts", lastHosts);
        context.workspaceState.update("lastBusbyHost", chosen);
    }
    return chosen;
}
exports.pickBusbyCore = pickBusbyCore;
//# sourceMappingURL=pickBusbyCore.js.map