"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickBusbyUrl = void 0;
const esb_1 = require("@busby/esb");
const vscode = require("vscode");
const vscode_1 = require("vscode");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickBusbyUrl(context) {
    let result = undefined;
    const quickPick = vscode_1.window.createQuickPick();
    quickPick.ignoreFocusOut = true;
    const urls = [];
    const lastUrl = context.workspaceState.get("lastBusbyUrl", "");
    if (lastUrl) {
        urls.push({
            label: "Last URL in Workspace",
            kind: vscode_1.QuickPickItemKind.Separator
        }, {
            label: lastUrl,
            picked: true,
        });
        quickPick.selectedItems = [urls[1]];
    }
    const lastUrls = context.globalState.get("lastBusbyUrls", []);
    if (lastUrls.length > 0) {
        for (const url of lastUrls) {
            if (url === lastUrl) {
                continue;
            }
            const item = {
                label: url,
                picked: !lastUrl && lastUrls.indexOf(url) === 0,
                buttons: [
                    {
                        iconPath: new vscode.ThemeIcon('close'),
                        tooltip: "Remove",
                    }
                ]
            };
            urls.push(item);
            if (!quickPick.selectedItems && !lastUrl && lastUrls.indexOf(url) === 0) {
                quickPick.selectedItems = [item];
            }
        }
    }
    quickPick.items = [...urls];
    quickPick.title = "Select or enter a selector/admin/config url";
    result = await new Promise((resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidChangeValue((value) => {
            if (!urls.find(h => h.label === value)) {
                quickPick.items = [{ label: quickPick.value, description: "New URL", }, ...urls];
            }
            else {
                quickPick.items = urls;
            }
        }), quickPick.onDidTriggerItemButton((e) => {
            urls.splice(urls.indexOf(e.item), 1);
            quickPick.items = urls;
            context.globalState.update("lastBusbyUrls", urls.filter(h => h.description !== "New URL" && h.kind !== vscode_1.QuickPickItemKind.Separator).map(h => h.label));
        }), quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            resolve(selection);
            disposable.dispose();
        }));
        quickPick.show();
    });
    const chosen = result?.label;
    if (chosen) {
        const lastUrls = context.globalState.get("lastBusbyUrls", []);
        if (lastUrls.includes(chosen)) {
            lastUrls.splice(lastUrls.indexOf(chosen), 1);
        }
        lastUrls.unshift(chosen);
        const config = vscode.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("core") ?? 0;
        if (history > 0) {
            if (lastUrls.length > history) {
                lastUrls.splice(history - 1, lastUrls.length - history);
            }
        }
        esb_1.Log.logInfo(`Last URL Changed to ${chosen}`);
        esb_1.Log.logInfo(`Last URLS Changed to ${lastUrls.length} ${lastUrls.join(", ")}`);
        context.globalState.update("lastBusbyUrls", lastUrls);
        context.workspaceState.update("lastBusbyUrl", chosen);
    }
    return chosen;
}
exports.pickBusbyUrl = pickBusbyUrl;
//# sourceMappingURL=pickBusbyUrl.js.map