"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickProject = void 0;
const vscode = require("vscode");
const vscode_1 = require("vscode");
const fs = require("fs/promises");
const path = require("path");
const esb_1 = require("@busby/esb");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickProject(context) {
    let result = undefined;
    let pickIndex = 2;
    const quickPick = vscode_1.window.createQuickPick();
    const projects = [
        { label: "all" },
    ];
    const lastProject = context.workspaceState.get("lastProject", "");
    esb_1.Log.logInfo(`Last Project ${lastProject}`);
    if (lastProject && typeof lastProject === "string") {
        if (lastProject === "all") {
            pickIndex = 0;
        }
        else {
            projects.push({
                label: "Last Project in Workspace",
                kind: vscode_1.QuickPickItemKind.Separator
            }, {
                label: lastProject,
                picked: true,
            });
        }
    }
    const lastProjects = context.globalState.get("lastProjects", []);
    esb_1.Log.logInfo(`Last Projects ${JSON.stringify(lastProjects)}`);
    if (lastProjects.length > 0) {
        projects.push({
            label: "Previous Projects",
            kind: vscode_1.QuickPickItemKind.Separator
        });
        for (const project of lastProjects) {
            if (project === lastProject) {
                continue;
            }
            if (project !== "all") {
                projects.push({
                    label: project,
                    picked: false,
                });
            }
        }
    }
    const config = vscode_1.workspace.getConfiguration("squaredpaper.repo");
    let workingFolder = path.join(config.get("busby_pspl", "/Users/michael/git/squared/busby_pspl"), "projects");
    const ws = vscode_1.workspace.workspaceFolders?.find(f => f.uri.path.includes("busby_pspl"));
    if (ws) {
        workingFolder = path.join(ws.uri.path.split("busby_pspl")[0], "busby_pspl", "projects");
    }
    try {
        const stat = await fs.stat(workingFolder);
        if (stat) {
            const folders = await fs.readdir(workingFolder);
            esb_1.Log.logDebug('Folders ' + folders.length);
            if (folders.length > 0) {
                projects.push({ kind: vscode_1.QuickPickItemKind.Separator, label: "All Projects" });
                for (const folder of folders) {
                    try {
                        const stat = await fs.stat(path.join(workingFolder, folder));
                        if (stat.isDirectory()) {
                            const label = path.basename(folder);
                            if (!lastProjects?.includes(label)) {
                                projects.push({ label });
                            }
                        }
                    }
                    catch (err) {
                        esb_1.Log.logError('Failed stating folder', err);
                    }
                }
                ;
            }
        }
        else {
            vscode_1.window.showErrorMessage(`Could not find any projects. Check your configuration. ${workingFolder}.`);
        }
    }
    catch (err) {
        esb_1.Log.logError('Failed getting working folder', err);
    }
    quickPick.items = projects;
    quickPick.activeItems = [projects[pickIndex]];
    quickPick.title = "Pick a busby project";
    result = await new Promise((resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.activeItems[0];
            resolve(selection);
            quickPick.hide();
            disposable.dispose();
        }));
        quickPick.show();
    });
    const chosen = result?.kind !== vscode_1.QuickPickItemKind.Separator ? result?.label : undefined;
    esb_1.Log.logInfo(`Project Chosen ${JSON.stringify(result)}`);
    esb_1.Log.logInfo(`Project Chosen ${chosen}`);
    if (chosen) {
        const lastProjects = context.globalState.get("lastProjects", []);
        esb_1.Log.logInfo(`Last Projects Current ${lastProjects.length} ${lastProjects.join(", ")}`);
        if (lastProjects.includes(chosen)) {
            lastProjects.splice(lastProjects.indexOf(chosen), 1);
        }
        lastProjects.unshift(chosen);
        esb_1.Log.logInfo(`Last Projects after ${lastProjects.length} ${lastProjects.join(", ")}`);
        const config = vscode_1.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("project") ?? 0;
        if (history > 0) {
            if (lastProjects.length > history) {
                lastProjects.splice(history - 1, lastProjects.length - history);
            }
        }
        esb_1.Log.logInfo(`Last Projects Changed to ${lastProjects.length} ${lastProjects.join(", ")}`);
        context.globalState.update("lastProjects", lastProjects);
        context.workspaceState.update("lastProject", chosen);
    }
    return chosen;
}
exports.pickProject = pickProject;
//# sourceMappingURL=pickProject.js.map