"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickService = void 0;
const node_1 = require("@busby/esb/node");
const _ = require("lodash");
const vscode = require("vscode");
const vscode_1 = require("vscode");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickService(context) {
    let result = undefined;
    const quickPick = vscode_1.window.createQuickPick();
    quickPick.ignoreFocusOut = true;
    const services = [];
    const lastServices = context.workspaceState.get("lastServices", []);
    if (lastServices.length > 0) {
        for (const service of lastServices) {
            services.push({
                label: service,
                picked: lastServices.indexOf(service) === 0,
                buttons: [
                    {
                        iconPath: new vscode.ThemeIcon('close'),
                        tooltip: "Remove",
                    }
                ]
            });
        }
        quickPick.activeItems = [services[0]];
        services.push({ kind: vscode_1.QuickPickItemKind.Separator, label: "" });
    }
    quickPick.items = [...services];
    quickPick.activeItems = [services[0]];
    quickPick.title = "Select a service";
    let manager = null;
    let subscriber = null;
    const disposables = [];
    result = await new Promise(async (resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidTriggerItemButton((e) => {
            services.splice(services.indexOf(e.item), 1);
            const current = quickPick.activeItems[0];
            quickPick.items = services;
            quickPick.activeItems = [current];
            context.workspaceState.update("lastServices", lastServices.filter(h => h !== e.item.label));
        }), quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            resolve(selection);
            disposable.dispose();
        }), {
            dispose: () => {
                for (const { dispose } of disposables) {
                    dispose?.();
                }
                subscriber?.close();
                manager?.closeAll();
            }
        });
        quickPick.show();
        const host = context.workspaceState.get("lastBusbyHost", "");
        if (host) {
            const previous = context.globalState.get(`busby.services.${host.replaceAll(".", "-")}`, []);
            const prevServices = [];
            for (const service of previous) {
                if (!services.find(p => p.label === service)) {
                    prevServices.push({
                        label: service,
                    });
                }
            }
            services.push(..._.sortBy(prevServices, "label"));
            const current = quickPick.activeItems[0];
            quickPick.items = services;
            quickPick.activeItems = [current];
            try {
                manager = node_1.SubscriberManager.getManager(node_1.LoginDetails.makeFromJson({
                    username: "squared",
                    password: "Squared1",
                    host,
                    providerId: host,
                }));
                subscriber = manager.getSubscriber("directory", node_1.BusbyDirectorySubscriber);
                const state = await Promise.race([
                    subscriber.control.getDirectoryState(undefined, { timeoutSecs: 5 }),
                    new Promise((resolve) => setTimeout(resolve, 10000))
                ]);
                if (state) {
                    const serviceIds = [];
                    const newServices = [];
                    const servicesResult = state.getResult().services ?? state.getResult();
                    for (const service of servicesResult) {
                        serviceIds.push(service.serviceId);
                        const current = services.find((s) => s.label === service.serviceId);
                        if (!current) {
                            newServices.push({
                                label: service.serviceId,
                            });
                        }
                    }
                    services.push(..._.sortBy(newServices, "label"));
                    context.globalState.update(`busby.services.${host.replaceAll(".", "-")}`, serviceIds);
                    disposables.push(subscriber.addStatusProcessor((msg) => {
                        console.log(msg.getParams());
                    }));
                }
                else {
                    console.warn("State not received");
                    vscode_1.window.showWarningMessage(`Could not find providers for host ${host}.`);
                }
                const current = quickPick.activeItems[0];
                quickPick.items = services;
                quickPick.activeItems = [current];
            }
            catch (err) {
                console.error("Error getting providers", err);
            }
            finally {
                // subscriber?.close();
                // manager?.closeAll();
            }
        }
    });
    const chosen = result?.label;
    if (chosen) {
        const lastServices = context.workspaceState.get("lastServices", []);
        if (lastServices.includes(chosen)) {
            lastServices.splice(lastServices.indexOf(chosen), 1);
        }
        lastServices.unshift(chosen);
        const config = vscode.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("service") ?? 0;
        if (history > 0) {
            if (lastServices.length > history) {
                lastServices.splice(history - 1, lastServices.length - history);
            }
        }
        node_1.Log.logInfo(`Last Services Changed to ${lastServices.length} ${lastServices.join(", ")}`);
        context.workspaceState.update("lastServices", lastServices);
    }
    return chosen;
}
exports.pickService = pickService;
//# sourceMappingURL=pickService.js.map