"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.shellDirect = void 0;
const node_1 = require("@busby/esb/node");
const vscode = require("vscode");
const uuid_1 = require("uuid");
async function shellDirect(context, location = vscode.TerminalLocation.Panel) {
    const ptyId = (0, uuid_1.v4)();
    const core = await vscode.commands.executeCommand("squaredpaper.pickBusbyCore");
    const node = await vscode.commands.executeCommand("squaredpaper.pickBusbyNode");
    if (!core || !node) {
        return;
    }
    const writeEmitter = new vscode.EventEmitter();
    const dimensionsEmitter = new vscode.EventEmitter();
    let subscriber;
    const onResize = (dimensions) => {
        subscriber.control.ptyResize({
            id: ptyId,
            data: { cols: dimensions.columns, rows: dimensions.rows },
        });
    };
    const onRemoteData = (data) => {
        const params = data.getParams();
        if (params.id === ptyId) {
            writeEmitter.fire(params.data);
        }
    };
    const onClose = (data) => {
        const params = data.getParams();
        if (params.id === ptyId) {
            writeEmitter.fire(`\n --- Remote session ended with code ${params.exit?.exitCode ?? 1} --- `);
            if (params.exit?.exitCode === 0) {
                process.exit(0);
            }
            else {
                // Allow some time for any data coming across
                setTimeout(() => process.exit(params.exit?.exitCode ?? 1), 2500);
            }
        }
    };
    const onLocalData = (data) => {
        subscriber.control.ptyData({
            id: ptyId,
            data,
        });
    };
    const onHeartbeat = async () => {
        await subscriber.control.ptyHeartbeat({
            id: ptyId,
        });
        setTimeout(onHeartbeat, 60 * 1000);
    };
    const terminal = vscode.window.createTerminal({
        name: node,
        location,
        pty: {
            onDidOverrideDimensions: dimensionsEmitter.event,
            onDidWrite: writeEmitter.event,
            close: () => { },
            handleInput: onLocalData,
            open: async (initial) => {
                const manager = node_1.SubscriberManager.getManager(node_1.LoginDetails.makeFromJson({
                    username: "squared",
                    password: "Squared1",
                    host: core,
                    providerId: core,
                }));
                subscriber = manager.getSubscriber("Monitor_" + node, node_1.MonitorSubscriber);
                await subscriber.waitUntilConnected();
                const request = await subscriber.control.getSystemHealth();
                const health = request.getResult();
                subscriber.status[`ptyData.${ptyId}`](onRemoteData);
                subscriber.status.ptyClose(onClose);
                const create = await subscriber.control.ptyStart({
                    id: ptyId,
                    shell: health.osType === "Windows_NT" ? (health.bash ? "C:\\opt\\squaredpaper\\git\\bin\\bash.exe" : health.wsl ? "bash.exe" : "powershell.exe") : "bash",
                    args: [],
                });
                if (create.isError()) {
                    console.error("Unable to initiate session");
                    terminal.dispose();
                    return;
                }
                onResize(initial);
                setTimeout(onHeartbeat, 60 * 1000);
            },
        }
    });
    terminal.show(false);
}
exports.shellDirect = shellDirect;
//# sourceMappingURL=shellDirect.js.map